package de.kreawi.trainer.db;

import javax.sql.DataSource;

import de.kreawi.trainer.onlinegame.OnlineGameSettings;
import de.kreawi.trainer.onlinegame.entities.OnlineGameUser;

public class TrainerOnlineGameSettings extends DbAccessModel implements OnlineGameSettings {
  
  private static final String KEY_LOGIN = "login";
  
  private static final String KEY_PASSWORD = "password";

  private static final String KEY_PROFILE_USER_ID = "profile_user_id";

  private static final String KEY_SERVER_SESSION = "server_session";

  private final TrainerOnlineGamePersistence persistence;

  public TrainerOnlineGameSettings(TrainerOnlineGamePersistence trainerOnlineGamePersistence, DataSource dataSource) {
    super(dataSource);
    
    this.persistence = trainerOnlineGamePersistence;
    
    runWithDbConn(() -> {
      TrainerOnlineGameSetting.createTableIfNotExists();
    });
    
  }

  @Override
  public String getLogin() {
    return getValue(KEY_LOGIN);
  }
  
  @Override
  public void setLogin(String login) {
    setValue(KEY_LOGIN, login);
  }
  
  @Override
  public String getPassword() {
    return getValue(KEY_PASSWORD);
  }
  
  @Override
  public void setPassword(String password) {
    setValue(KEY_PASSWORD, password);
  }
  
  @Override
  public OnlineGameUser getProfileUser() {
    final String profileUserId = getValue(KEY_PROFILE_USER_ID);
    if (profileUserId == null) {
      return null;
    }
    return this.persistence.getUser(profileUserId);
  }
  
  @Override
  public void updateProfileUser(final OnlineGameUser user) {
    this.persistence.updateOrInsertUser(user);
    setValue(KEY_PROFILE_USER_ID, user.getId());
  }
  
  @Override
  public String getServerSession() {
    return getValue(KEY_SERVER_SESSION);
  }
  
  @Override
  public void setServerSession(String session) {
    setValue(KEY_SERVER_SESSION, session);
  }
  
  private void setValue(final String key, final String value) {
    runWithDbConn(() -> {
      final TrainerOnlineGameSetting s = TrainerOnlineGameSetting.findOrInit(TrainerOnlineGameSetting.DB_KEY, key);
      s.setValue(value);
      s.saveIt();
    });
  }
  
  private String getValue(final String key) {
    return callWithDbConn(() -> {
      final TrainerOnlineGameSetting s = TrainerOnlineGameSetting.findFirst(TrainerOnlineGameSetting.DB_KEY + "=?", key);
      return (s == null?null:s.getValue());
    });
  }

}
