package de.kreawi.trainer.dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.uif_lite.component.Factory;

@SuppressWarnings("serial")
public abstract class KreawiDialogPanel extends JPanel {

	private static final int SPLIT_INSET = 3;

	private ArrayList<JSplitPane> splitPanes = new ArrayList<JSplitPane>();

	private ComponentAdapter splitPaneComponentAdapter = null;

	private ComponentAdapter focusComponentAdapter = null;

	protected JComponent focusJComponent = null;

  private static final Logger logger = LoggerFactory.getLogger(KreawiDialogPanel.class); 

	protected void setSplitPaneDividerCorrection(boolean enabled) {

		if (enabled) {
			this.splitPaneComponentAdapter = new ComponentAdapter() {

				public void componentShown(ComponentEvent e) {
					adjustSplitDividerLocations();
					// if (focusJComponent != null) {
					// focusJComponent.requestFocusInWindow();
					// }
				}

				public void componentResized(ComponentEvent e) {
					adjustSplitDividerLocations();
				}

			};
			addComponentListener(this.splitPaneComponentAdapter);
		} else {
			removeComponentListener(this.splitPaneComponentAdapter);
			this.splitPaneComponentAdapter = null;
		}
	}

	protected void setFocusJComponent(JComponent comp) {

		if (this.focusComponentAdapter == null) {
			this.focusComponentAdapter = new ComponentAdapter() {
				public void componentShown(ComponentEvent e) {
					if (focusJComponent != null) {
						focusJComponent.requestFocusInWindow();
					}
				}
			};
			addComponentListener(this.focusComponentAdapter);
		}

		this.focusJComponent = comp;
	}

	/**
	 * Builds and returns the horizontal split using stripped split panes.
	 */
	protected JSplitPane buildHorizontalSplit(Component left, Component right,
			double resizeWeight) {
		JSplitPane pane = Factory.createStrippedSplitPane(
				JSplitPane.HORIZONTAL_SPLIT, left, right, resizeWeight);
		pane.setOpaque(false);
		addSplitPaneToAdjust(pane);
		pane.setDividerSize(SPLIT_INSET * 2);

		return pane;
	}

	/**
	 * Builds and returns the vertical split using stripped split panes.
	 * <p>
	 */
	protected JSplitPane buildVerticalSplit(Component upper, Component lower,
			double resizeWeight) {
		JSplitPane pane = Factory.createStrippedSplitPane(
				JSplitPane.VERTICAL_SPLIT, upper, lower, resizeWeight);
		pane.setOpaque(false);
		addSplitPaneToAdjust(pane);
		pane.setDividerSize(SPLIT_INSET * 2);

		return pane;
	}

	/**
	 * Builds and returns a static horizontal split using JPanels with the split
	 * exactly in the middle.
	 */
	protected JPanel buildStaticHorizontalSplit(Component left, Component right) {
		JPanel panel = new JPanel(new GridLayout(1, 2, SPLIT_INSET * 2, 0));
		panel.setOpaque(false);

		panel.add(left);
		panel.add(right);

		return panel;
	}

	/**
	 * Builds and returns a static horizontal split using JPanels with the split
	 * exactly in the middle..
	 */
	protected JPanel buildStaticVerticalSplit(Component upper, Component lower) {
		JPanel panel = new JPanel(new GridLayout(2, 1, 0, SPLIT_INSET * 2));
		panel.setOpaque(false);

		panel.add(upper);
		panel.add(lower);

		return panel;
	}

	/**
	 * Builds and returns a static horizontal split using JPanels.
	 */
	protected JPanel buildStaticHorizontalSplit(Component left,
			Component right, int percent) {

		return buildStaticHorizontalSplit(left, right, percent,
				((double) percent / 100d));
	}
	
	/**
	 * Builds and returns a static horizontal split using JPanels.
	 */
	protected JPanel buildStaticHorizontalSplit(Component left,
			Component right, int percent, double resizeWeight) {
		JPanel panel = new JPanel(new GridBagLayout());
		panel.setOpaque(false);

		GridBagConstraints gbc;
		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0, 0, 0, SPLIT_INSET);
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = percent;
		gbc.weightx = resizeWeight;
		gbc.weighty = 1;

		panel.add(left, gbc);

		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0, SPLIT_INSET, 0, 0);
		gbc.gridy = 0;
		gbc.gridx = percent;
		gbc.gridheight = 1;
		gbc.gridwidth = 100 - percent;
		gbc.weightx = 1 - resizeWeight;
		gbc.weighty = 1;

		panel.add(right, gbc);

		return panel;
	}

	/**
	 * Builds and returns a static horizontal split using JPanels.
	 */
	protected JPanel buildStaticVerticalSplit(Component upper, Component lower,
			int percent) {
		
		return buildStaticVerticalSplit(upper, lower, percent, ((double)percent / 100d));
	}
	
	/**
	 * Builds and returns a static horizontal split using JPanels.
	 */
	protected JPanel buildStaticVerticalSplit(Component upper, Component lower,
			int percent, double resizeWeight) {
		JPanel panel = new JPanel(new GridBagLayout());
		panel.setOpaque(false);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0, 0, SPLIT_INSET, 0);
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.gridheight = percent;
		gbc.gridwidth = 1;
		gbc.weightx = 1;
		gbc.weighty = resizeWeight;

		panel.add(upper, gbc);

		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(SPLIT_INSET, 0, 0, 0);
		gbc.gridy = percent;
		gbc.gridx = 0;
		gbc.gridheight = 100 - percent;
		gbc.gridwidth = 1;
		gbc.weightx = 1;
		gbc.weighty = 1 - resizeWeight;

		panel.add(lower, gbc);

		return panel;
	}

	private void addSplitPaneToAdjust(JSplitPane splitPane) {
		this.splitPanes.add(splitPane);
	}

	public void adjustSplitDividerLocations() {
		for (JSplitPane split : this.splitPanes) {
			adjustSplitDividerLocation(split);
		}
	}

	private void adjustSplitDividerLocation(JSplitPane split) {
		split.setDividerLocation(split.getResizeWeight());
	}

//	protected static void setTextPaneFont(JTextPane jtp, Font font) {
//		// Start with the current input attributes for the JTextPane. This
//		// should ensure that we do not wipe out any existing attributes
//		// (such as alignment or other paragraph attributes) currently
//		// set on the text area.
//		MutableAttributeSet attrs = jtp.getInputAttributes();
//	
//		// Set the font family, size, and style, based on properties of
//		// the Font object. Note that JTextPane supports a number of
//		// character attributes beyond those supported by the Font class.
//		// For example, underline, strike-through, super- and sub-script.
//		StyleConstants.setFontFamily(attrs, font.getFamily());
//	
//		// Retrieve the pane's document object
//		StyledDocument doc = jtp.getStyledDocument();
//	
//		// Replace the style for the entire document. We exceed the length
//		// of the document by 1 so that text entered at the end of the
//		// document uses the attributes.
//		doc.setCharacterAttributes(0, doc.getLength() + 1, attrs, false);
//	}

}
