package de.kreawi.trainer.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Map;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.builder.ButtonStackBuilder;
import com.jgoodies.forms.factories.Borders;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerMain;
import de.kreawi.trainer.util.TrainerHtmlUtil;
import de.kreawi.trainer.util.TrainerTextResources;
import de.martinknauer.util.Util;

@SuppressWarnings("serial")
public class MainDialog extends KreawiDialogPanel {

	public static final String LOGO_PATH = KreawiGlobals.APP_IMAGES_DIR_NAME + "/logo_nur kreis_transparent_sw.gif";

	private Map<Actions, Action> actionMap;

  private static final Logger logger = LoggerFactory.getLogger(MainDialog.class); 

	/**
	 * Builds the dialog.
	 */
	public MainDialog(Map<Actions, Action> actionMap) {

		this.actionMap = actionMap;

		setLayout(new GridBagLayout());
		setOpaque(false);
		setBorder(Borders.DIALOG_BORDER);

		JPanel panel = new JPanel(new GridBagLayout());
		panel.setOpaque(false);
		
		Dimension defaultDialogSize = Util.isExtraLargeFontSize() ? TrainerFrame.MEDIUM_DIALOG_SIZE
				: TrainerFrame.SMALL_DIALOG_SIZE;

		int width = defaultDialogSize.width - 40;
		int height = defaultDialogSize.height - 60;
		
		final Dimension dialogSize = new Dimension(width, height);
    panel.setPreferredSize(dialogSize);
		panel.setMinimumSize(panel.getPreferredSize());
		panel.setMaximumSize(dialogSize);

		panel.setBorder(BorderFactory.createEmptyBorder());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		JPanel horizontalSplit = buildStaticHorizontalSplit(
				buildWrittenExamsPanel(), buildOralExamsPanel());

		JPanel verticalSplit = buildStaticVerticalSplit(horizontalSplit,
				buildLowerPanel(), 85);

		panel.add(verticalSplit, gbc);
		add(panel);
	}

  private JComponent buildWrittenExamsPanel() {
    return buildSelectionPanel(TrainerTextResources.getString("WRITTEN_EXAMINATION_TITLE"),
        TrainerTextResources.getString("WRITTEN_EXAMINATION_EXPLANATION"), this.actionMap.get(Actions.MC_RANDOM_QUIZ),
        this.actionMap.get(Actions.WRITTEN_BY_CATEGORY), this.actionMap.get(Actions.WRITTEN_BY_EXAM),
        this.actionMap.get(Actions.MC_INTENSIVE_CARE), this.actionMap.get(Actions.MC_HISTORY));
  }

  private JComponent buildOralExamsPanel() {
    return buildSelectionPanel(TrainerTextResources.getString("ORAL_EXAMINATION_TITLE"),
        TrainerTextResources.getString("ORAL_EXAMINATION_EXPLANATION"), this.actionMap.get(Actions.TEXT_RANDOM_QUIZ),
        this.actionMap.get(Actions.ORAL_BY_CATEGORY), this.actionMap.get(Actions.ORAL_BY_EXAM),
        this.actionMap.get(Actions.TEXT_INTENSIVE_CARE), this.actionMap.get(Actions.TEXT_HISTORY));
  }

  private JComponent buildSelectionPanel(final String title, final String explanation, final Action randomQuizAction,
      final Action categoryAction, final Action examAction, final Action icuAction, final Action historyAction) {

		KreawiInternalFrame kif = new KreawiInternalFrame(title,
				new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		JPanel gradientPanel = kif.getContent();

		JLabel label = new JLabel(explanation);
		JLabel selectionLabel = new JLabel(TrainerTextResources
				.getString("SELECTION_METHOD_TITLE"));
		JButton randomQuizButton = new JButton(randomQuizAction);
		JButton categoryButton = new JButton(categoryAction);
		JButton examButton = new JButton(examAction);
		JButton icuButton = new JButton(icuAction);
		JButton historyButton = new JButton(historyAction);

		ButtonStackBuilder builder = new ButtonStackBuilder();
		builder.addGridded(selectionLabel);
    JButton[] buttons = { categoryButton, examButton };
    builder.addGridded(randomQuizButton);
    builder.addUnrelatedGap();
		builder.addButtons(buttons);
		builder.addUnrelatedGap();
		builder.addGridded(icuButton);
		builder.addRelatedGap();
		builder.addGridded(historyButton);

		JPanel labelPanel = new JPanel(new GridBagLayout());
		labelPanel.setOpaque(false);
		label.setOpaque(false);
		labelPanel.add(label);

		JPanel buttonPanel = builder.getPanel();
		buttonPanel.setOpaque(false);

		JPanel selectionPanel = new JPanel(new FlowLayout());
		selectionPanel.setOpaque(false);
		selectionLabel.setOpaque(false);
		selectionPanel.add(builder.getPanel());

		gradientPanel.add(labelPanel, gbc);
		
		gbc.gridy = 1;
		gbc.weighty = 0.5;
		
		gradientPanel.add(selectionPanel, gbc);

		return kif;
	}

	private JComponent buildLowerPanel() {
		return MainDialog.buildAppTitlePanel(true);
	}

	public static KreawiInternalFrame buildAppTitlePanel(boolean drawShadow) {

		KreawiInternalFrame sif;
		
		if (drawShadow) {
			sif = new KreawiInternalFrame(null, new BorderLayout());
		} else {
			sif = new KreawiInternalFrame(null, new BorderLayout(), null, true, false, false);
		}

		JPanel gradientPanel = sif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder(3, 4, 3, 4));

		JPanel labelPanel = new JPanel(new BorderLayout());
		labelPanel.setOpaque(false);
		JPanel innerPanel = new JPanel(new FlowLayout());
		innerPanel.setOpaque(false);
		final String footText;
    if (KreawiGlobals.isDemo() && drawShadow) {
		  final String url = TrainerTextResources.getString("APPLICATION_TITLE_FOOT_BUY_URL");
		  final String[] subs = new String[] {
		      KreawiGlobals.CURRENT_YEAR, 
		      KreawiGlobals.CURRENT_YEAR,
		      url,
		      url
      };
      footText = TrainerTextResources.getStringWithSubstitution("APPLICATION_TITLE_FOOT_DEMO", subs);
    } else {
      footText = TrainerTextResources.getStringWithSubstitution("APPLICATION_TITLE_FOOT", new String[] {
          KreawiGlobals.CURRENT_YEAR, KreawiGlobals.CURRENT_YEAR });
    }
    final JScrollPane textLabel = TrainerHtmlUtil.createHTMLMessageScrollPane(footText);
    textLabel.setBorder(BorderFactory.createEmptyBorder());
    textLabel.setOpaque(false);
    textLabel.getViewport().setOpaque(false);
    // Overwrite default behaviour to always show scroll bar (only for mac)!
    textLabel.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
    textLabel.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    innerPanel.add(textLabel);
    if (drawShadow) {
      JLabel imageLabel = new JLabel(new ImageIcon(TrainerMain.class.getResource(LOGO_PATH)));
      innerPanel.add(imageLabel);
    }
    innerPanel.setBackground(Color.GREEN);
		labelPanel.add(innerPanel, BorderLayout.EAST);

		gradientPanel.add(labelPanel, BorderLayout.SOUTH);

		return sif;
	}
	
}
