package de.kreawi.trainer.dialogs.backup;

import java.io.File;
import java.io.Serializable;
import java.text.DateFormat;

import de.kreawi.trainer.util.TrainerTextResources;

public class TrainerBackup implements Serializable {
  
  private static final long serialVersionUID = 2391690884465704732L;
  
  private String title;
  private long time;
  private File backupDir;

  private final boolean dbBackup;

  private final boolean otherTrainer;


  public TrainerBackup(final String backupTitle, final long time, final File backupDir) {
    this(backupTitle, time, backupDir, true, false);
  }
  
  public TrainerBackup(final String backupTitle, final long time, final File backupDir, final boolean dbBackup,
      final boolean otherTrainer) {
    this.title = backupTitle;
    this.time = time;
    this.backupDir = backupDir;
    this.dbBackup = dbBackup;
    this.otherTrainer = otherTrainer;
  }

  public String getTitle() {
    return title;
  }

  public long getTime() {
    return time;
  }
  
  public String toString() {
    final String timeString = getDateTimeString(this.time);
    if (title == null || "".equals(title)) {
      return TrainerTextResources.getStringWithSubstitution("RESTORE_USER_DATA_DEFAULT_TITLE", timeString);
    } else {
      return title + " (" + timeString + ")";
    }
  }
  
  private String getDateTimeString(Long timestamp) {
    return DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.SHORT).format(timestamp);
  }

  public File getBackupDir() {
    return backupDir;
  }

  public boolean isDbBackup() {
    return this.dbBackup;
  }

  public boolean isOtherTrainer() {
    return this.otherTrainer;
  }
  
}
