package de.kreawi.trainer.dialogs.history;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerModel;

public abstract class AbstractHistoryTableModel extends AbstractTableModel {

	private static final long serialVersionUID = -7485148038100775437L;

	public static final String ICON_EXAM_MODE = KreawiGlobals.APP_IMAGES_DIR_NAME + "/status_display_grey_ok.png";

	protected TrainerModel model;

  private static final Logger logger = LoggerFactory.getLogger(AbstractHistoryTableModel.class); 

	public AbstractHistoryTableModel(final TrainerModel model) {
		this.model = model;
	}

	public int getRowCount() {
		return getHistoryItems().size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {

	  return getValue(rowIndex, columnIndex);

	}

	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
	  throw new IllegalArgumentException("Only deletion (setting to null) is allowed!");
	}
	
	public void removeRows(int[] indices) {
    ArrayList<HistoryItem> itemsToDelete = new ArrayList<HistoryItem>();
    Arrays.sort(indices);
    for (int i = indices.length - 1; i >= 0; i--) {
      itemsToDelete.add(getHistoryItems().get(indices[i]));
    }
    for (HistoryItem item: itemsToDelete) {
      item.delete();
    }
    fireTableRowsDeleted(indices[0], indices[indices.length - 1]);
	}

	protected long getStartTimeTimestamp(final int rowIndex) {
	  return getHistoryItems().get(rowIndex).getQuiz().getStartTime();
  }
	
	abstract List<HistoryItem> getHistoryItems();
	
	protected abstract Object getValue(int rowIndex, int columnIndex);

  @SuppressWarnings("rawtypes")
  public abstract Comparator getComparator(int columnIndex);

}
