package de.kreawi.trainer.dialogs.history;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.text.BadLocationException;

import org.jdesktop.swingx.JXTable;
import org.jdesktop.swingx.decorator.SortOrder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.builder.ButtonBarBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.uif_lite.component.Factory;
import com.jquizengine.questions.collection.QuestionPool;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerSettingsConstants;
import de.kreawi.trainer.dialogs.KreawiDialogPanel;
import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import de.kreawi.trainer.quiz.KreawiQuiz;
import de.kreawi.trainer.util.OSDetector;
import de.kreawi.trainer.util.TrainerTextResources;
import de.kreawi.trainer.util.swingx.CellStringFiltering;

@SuppressWarnings("serial")
public abstract class HistoryDialog extends KreawiDialogPanel {

	protected TrainerModel model;

	protected Map<Actions, Action> actionMap;
	protected JLabel numberOfTotalQuestionsLabel;
	protected JLabel numberOfSelectedQuestionsLabel;
	protected JButton loadButton;
	protected JButton deleteButton;
	protected QuestionPool allQuestions;
	protected JLabel explanationLabel;
	protected TrainerFrame mainframe;
	protected HistoryTable table;

	private boolean needsRepack = false;

	private AbstractAction loadAction;

	private AbstractAction deleteAction;

	private static final Logger logger = LoggerFactory.getLogger(HistoryDialog.class); 

  public HistoryDialog(TrainerFrame mainframe, TrainerModel model, Map<Actions, Action> actionMap) {

		this.mainframe = mainframe;
		this.model = model;
		this.actionMap = actionMap;
		
		initTable();
		initActions();

		setLayout(new GridBagLayout());
		setOpaque(false);
		setBorder(Borders.DIALOG_BORDER);

		JPanel panel = new JPanel(new GridBagLayout());
		panel.setOpaque(false);
		
		int width = TrainerFrame.SMALL_DIALOG_SIZE.width - 40;
		int height = TrainerFrame.SMALL_DIALOG_SIZE.height - 90;

		// set the panel's preferred size so the table won't grow too big in non
		// maximized state
		panel.setPreferredSize(new Dimension(width, height));
		panel.setMinimumSize(panel.getPreferredSize());

		panel.setBorder(BorderFactory.createEmptyBorder());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		panel.add(buildStaticVerticalSplit(buildStaticVerticalSplit(buildFilterPanel(), buildHistoryPanel(), 5),
				buildButtonsPanel(), 100), gbc);
		add(panel, gbc);

		addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent e) {

				table.revalidate();
				table.repaint();

				if (table.getRowCount() < 1) {
					needsRepack = true;
				} else if (needsRepack) {
					table.packAll();
				}

				showExplanationMessage();

			}
		});
	}

	protected KreawiInternalFrame buildFilterPanel() {

		KreawiInternalFrame kif = new KreawiInternalFrame(null,
				new GridBagLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel
				.setBorder(BorderFactory.createEmptyBorder(6, 10, 7, 10));
		
		JPanel filterPanel = new JPanel(new GridLayout(1,3));
		filterPanel.setOpaque(false);
		filterPanel.setBorder(BorderFactory.createEmptyBorder());
		
		/* filter textfield begin */
		JPanel outerFieldPanel = new JPanel(new BorderLayout());
		outerFieldPanel.setOpaque(false);
		outerFieldPanel.setBorder(BorderFactory.createEmptyBorder());
		
		JPanel innerFieldPanel = new JPanel(new BorderLayout(2, 2));
		innerFieldPanel.setOpaque(false);
		innerFieldPanel.setBorder(BorderFactory.createEmptyBorder());
		
		JLabel filterLabel = new JLabel(TrainerTextResources.getString("HISTORY_LABEL_FILTER") + ":");
		
		final JTextField filterField = new JTextField(24);
		
		final CellStringFiltering filter = new CellStringFiltering(this.table, 2);
		
		filterField.getDocument().addDocumentListener(new DocumentListener() {

			public void changedUpdate(DocumentEvent e) {
				handleEvent(e);
			}

			public void insertUpdate(DocumentEvent e) {
				handleEvent(e);
			}

			public void removeUpdate(DocumentEvent e) {
				handleEvent(e);
			}
			
			private void handleEvent(DocumentEvent e) {
				try {
					filter.setFilterString(e.getDocument().getText(0, e.getDocument().getLength()));
				} catch (BadLocationException ble) {
					// do nothing
				}
			}
			
		});
		
		String icon = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2.gif";
		String iconRollover = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2_light.gif";
		String iconPressed = KreawiGlobals.APP_IMAGES_DIR_NAME + "/close12_grey2_dark.gif";

		URL url = KreawiGlobals.class.getResource(icon);
		URL urlRollover = KreawiGlobals.class.getResource(iconRollover);
		URL urlPressed = KreawiGlobals.class.getResource(iconPressed);

		ActionListener listener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				filterField.setText("");
			}
		};

		String toolTip = TrainerTextResources.getString("HISTORY_TEXT_ERASE_FILTER");
		
		JButton resetButton = KreawiInternalFrame.createIconButton(
				new ImageIcon(url), new ImageIcon(urlRollover), new ImageIcon(
						urlPressed), toolTip, listener);
		
		innerFieldPanel.add(filterLabel, BorderLayout.WEST);
		innerFieldPanel.add(filterField, BorderLayout.CENTER);
		innerFieldPanel.add(resetButton, BorderLayout.EAST);
		
		outerFieldPanel.add(innerFieldPanel, BorderLayout.CENTER);
		
		/* filter textfield end */

		/* number in view label begin */
		JPanel viewPanel = new JPanel(new BorderLayout());
		viewPanel.setOpaque(false);
		viewPanel.setBorder(BorderFactory.createEmptyBorder());

		final JLabel viewLabel = new JLabel(TrainerTextResources
				.getStringWithSubstitution("HISTORY_LABEL_NUMBER_IN_VIEW",
						filter.getViewSize() + ""), JLabel.CENTER);
		filter.addPropertyChangeListener("filterString",
				new PropertyChangeListener() {

					public void propertyChange(PropertyChangeEvent evt) {
						viewLabel.setText(TrainerTextResources
								.getStringWithSubstitution(
										"HISTORY_LABEL_NUMBER_IN_VIEW", filter
												.getViewSize()
												+ ""));
					}

				});
		
		this.table.getModel().addTableModelListener(new TableModelListener() {

			public void tableChanged(TableModelEvent e) {
				viewLabel.setText(TrainerTextResources
						.getStringWithSubstitution(
								"HISTORY_LABEL_NUMBER_IN_VIEW", filter
										.getViewSize()
										+ ""));
			}
			
		});

		viewPanel.add(viewLabel, BorderLayout.CENTER);
		/* number in view label end */

		/* total number label begin */
		JPanel totalPanel = new JPanel(new BorderLayout());
		totalPanel.setOpaque(false);
		totalPanel.setBorder(BorderFactory.createEmptyBorder());

		final JLabel totalLabel = new JLabel(TrainerTextResources
				.getStringWithSubstitution("HISTORY_LABEL_NUMBER_TOTAL",
						this.table.getRowCount() + ""), JLabel.CENTER);
		
		this.table.getModel().addTableModelListener(new TableModelListener() {

			public void tableChanged(TableModelEvent e) {
				totalLabel.setText(TrainerTextResources
						.getStringWithSubstitution(
								"HISTORY_LABEL_NUMBER_TOTAL", table.getModel()
										.getRowCount()
										+ ""));
			}

		});

		totalPanel.add(totalLabel, BorderLayout.CENTER);
		/* total number label end */
		
		filterPanel.add(outerFieldPanel);
		filterPanel.add(viewPanel);
		filterPanel.add(totalPanel);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		gradientPanel.add(filterPanel, gbc);

		return kif;
	}
	
	private void initActions() {
		
		InputMap inputMap = this.table.getInputMap();
		ActionMap actionMap = this.table.getActionMap();
		
		/* popup begin */
		final JPopupMenu popup = new JPopupMenu();

		this.table.addMouseListener(new MouseAdapter() {

			public void mousePressed(MouseEvent e) {
				handlePopup(e);
				e.consume();
			}

			public void mouseReleased(MouseEvent e) {
				handlePopup(e);
				e.consume();
			}

			void handlePopup(MouseEvent e) {
				if (e.isPopupTrigger()) {
					popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}

		});
		/* popup end */
		
		/* load action begin */
		this.loadAction = new AbstractAction(TrainerTextResources
				.getString("HISTORY_ACTION_LOAD")) {

					public void actionPerformed(ActionEvent e) {
						selectionConfirmed();
					}
			
		};
		
		KeyStroke key;
		key = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);
		loadAction.putValue(Action.ACCELERATOR_KEY, key);
		loadAction.putValue(Action.ACTION_COMMAND_KEY, "loadQuiz");
		loadAction.setEnabled(false);
		inputMap.put(key, loadAction.getValue(Action.ACTION_COMMAND_KEY));
		actionMap.put(loadAction.getValue(Action.ACTION_COMMAND_KEY), loadAction);
		popup.add(new JMenuItem(loadAction));
		/* load action end */
		
		/* delete action begin */
		this.deleteAction = new AbstractAction(TrainerTextResources
				.getString("HISTORY_ACTION_DELETE")) {

					public void actionPerformed(ActionEvent e) {
						deleteConfirmed();
					}
			
		};
		
		if (OSDetector.getOs().isMac()) {
      /* mac os */
      key = KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, Event.META_MASK);
    } else {
			/* non-mac os */
			key = KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0);
		}
		deleteAction.putValue(Action.ACCELERATOR_KEY, key);
		deleteAction.putValue(Action.ACTION_COMMAND_KEY, "deleteQuiz");
		deleteAction.setEnabled(false);
		inputMap.put(key, deleteAction.getValue(Action.ACTION_COMMAND_KEY));
		actionMap.put(deleteAction.getValue(Action.ACTION_COMMAND_KEY), deleteAction);
		popup.add(new JMenuItem(deleteAction));
		/* delete action end */
		
	}
	
	protected KreawiInternalFrame buildButtonsPanel() {

		KreawiInternalFrame kif = new KreawiInternalFrame(null,
				new GridBagLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel
				.setBorder(BorderFactory.createEmptyBorder(10, 10, 11, 10));

		JButton loadButton = new JButton();
		loadButton.setAction(this.loadAction);
		loadButton.setText(TrainerTextResources
				.getString("HISTORY_BUTTON_LOAD"));

		JButton deleteButton = new JButton();
		deleteButton.setAction(deleteAction);
		deleteButton.setText(TrainerTextResources
				.getString("HISTORY_BUTTON_DELETE"));

		JButton backButton = new JButton(this.actionMap
				.get(Actions.BACK_TO_MAIN_DIALOG));

		ButtonBarBuilder builder = new ButtonBarBuilder();

		if (builder.isLeftToRightButtonOrder()) {
			builder.addGlue();
			builder.addGridded(loadButton);
			builder.addGlue();
			builder.addGlue();
			builder.addGridded(deleteButton);
			builder.addGlue();
			builder.addGlue();
			builder.addGridded(backButton);
			builder.addGlue();
		} else {
			builder.addGlue();
			builder.addGridded(backButton);
			builder.addGlue();
			builder.addGlue();
			builder.addGridded(deleteButton);
			builder.addGlue();
			builder.addGlue();
			builder.addGridded(loadButton);
			builder.addGlue();
		}

		JPanel buttonsPanel = builder.getPanel();
		buttonsPanel.setOpaque(false);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		gradientPanel.add(buttonsPanel, gbc);

		return kif;
	}

	protected KreawiInternalFrame buildHistoryPanel() {

		KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
				.getString("HISTORY_TABLE_TITLE"), new BorderLayout());
		
		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());

		JScrollPane scrollPane = Factory.createStrippedScrollPane((this.table));

		if (Boolean.TRUE.equals(UIManager.getDefaults().get(
				KreawiInternalFrame.USE_GRADIENT_BACKGROUND_KEY))) {
			scrollPane.setOpaque(false);
			scrollPane.getViewport().setOpaque(false);
		}

		gradientPanel.add(scrollPane);
		
		setFocusJComponent(this.table);
		
		return kif;
	}
	
	private void initTable() {
		final AbstractHistoryTableModel historyTableModel = getHistoryTableModel();
    this.table = new HistoryTable(historyTableModel);
		this.table.putClientProperty(JXTable.USE_DTCR_COLORMEMORY_HACK, Boolean.FALSE);
		this.table.setDefaultRenderer(this.table.getColumnClass(0),
				new HistoryCellRenderer());
		this.table.setBorder(BorderFactory.createEmptyBorder(0, 2, 0, 0));

		this.table.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						tableSelectionChanged();
					}
				});

		this.table.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent me) {
				if (me.getClickCount() == 2) {
					selectionConfirmed();
				}
			}
		});
		for (int i=0; i<this.table.getColumnCount(); i++) {
	    this.table.getColumnExt(i).setComparator(historyTableModel.getComparator(i));
		}

		this.table.setSortOrder(0, SortOrder.DESCENDING);
		
		this.table.setOpaque(false);
		this.table.setAutoResizeMode(JXTable.AUTO_RESIZE_ALL_COLUMNS);

		this.table.packAll();
	}

	private void tableSelectionChanged() {
		this.loadAction.setEnabled(this.table.getSelectedRowCount() == 1);
		this.deleteAction.setEnabled(this.table.getSelectedRowCount() > 0);
	}

  protected void selectionConfirmed() {
    final int selectedRow = this.table.getSelectedRow();
    if (selectedRow == -1) {
      // if called through double click there maybe no selected row!!!
      return;
    }
    final Object obj = this.table.getModel().getValueAt(this.table.convertRowIndexToModel(selectedRow), 0);
    if (obj instanceof KreawiQuiz) {
      final KreawiQuiz quiz = (KreawiQuiz) obj;
      if (quiz.getQuestionCount() > 0) {
        this.model.setSelectedQuestions(quiz);
      } else {
        final String message = TrainerTextResources.getString("HISTORY_TABLE_NO_QUESTIONS_MESSAGE");
        final String title = TrainerTextResources.getString("HISTORY_TABLE_NO_QUESTIONS_TITLE");
        this.mainframe.showMessageDialog(message, title, JOptionPane.WARNING_MESSAGE);
      }
    }
  }

	protected void deleteConfirmed() {
	  logger.info("this.table.getSelectedRowCount(): " + this.table.getSelectedRowCount());
	  final int[] indicesTable = this.table.getSelectedRows();
	  final int[] indicesModel = new int[indicesTable.length];
	  for (int i=0; i < indicesTable.length; i++) {
	    indicesModel[i] = this.table.convertRowIndexToModel(indicesTable[i]);
	  }
	  
	  this.table.getModel().removeRows(indicesModel);
	  logger.info("this.table.getSelectedRowCount(): " + this.table.getSelectedRowCount());
	}

  protected abstract AbstractHistoryTableModel getHistoryTableModel();

	protected void showExplanationMessage() {
		this.model.getHelpRegistry().showHelpMessage(this.mainframe,
				TrainerSettingsConstants.KEY_SHOW_HISTORY_EXPLANATION_MESSAGE);
	}

}