package de.kreawi.trainer.dialogs.history;

import java.util.Comparator;
import java.util.List;

import javax.swing.ImageIcon;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.TrainerMain;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerStorage.McHistoryChangedListener;
import de.kreawi.trainer.quiz.KreawiQuiz;
import de.kreawi.trainer.quiz.KreawiQuizHelper;
import de.kreawi.trainer.util.TrainerTextResources;

public class McQuizHistoryTableModel extends AbstractHistoryTableModel {

	private static final long serialVersionUID = 4389788471198025204L;

	private final static String[] COLUMNS = {
			TrainerTextResources.getString("HISTORY_COLUMN_START"),
			TrainerTextResources.getString("HISTORY_COLUMN_MODIFIED"),
			TrainerTextResources.getString("HISTORY_COLUMN_CATEGORY_EXAM"),
			TrainerTextResources.getString("HISTORY_COLUMN_QUESTIONS"),
			TrainerTextResources.getString("HISTORY_COLUMN_DONE"),
			TrainerTextResources.getString("HISTORY_COLUMN_CORRECT"),
			TrainerTextResources.getString("HISTORY_COLUMN_TIME"),
			TrainerTextResources.getString("HISTORY_COLUMN_EXAMMODE") };

	private static final Logger logger = LoggerFactory.getLogger(McQuizHistoryTableModel.class); 

	public McQuizHistoryTableModel(final TrainerModel model) {
		super(model);
		model.getStorage().addMcHistoryChangedListener(new McHistoryChangedListener() {
      @Override
      public void historyChanged() {
        fireTableDataChanged();
      }
    });
	}

	public int getColumnCount() {
		return COLUMNS.length;
	}

	@Override
	public String getColumnName(int column) {
		return COLUMNS[column];
	}

	List<HistoryItem> getHistoryItems() {
	   return model.getHistoryItemsMc();
	}

  @SuppressWarnings("rawtypes")
  @Override
  public Comparator getComparator(final int columnIndex) {
    switch (columnIndex) {
    case 0:
      return new QuizComparator();
    default:
      return new Comparator<Object>() {
        public int compare(Object o1, Object o2) {
          return o1.toString().compareTo(o2.toString());
        }
      };
    }
  }

  @Override
	protected Object getValue(final int rowIndex, int columnIndex) {
    
    final KreawiQuiz quiz = getHistoryItems().get(rowIndex).getQuiz();
    
		switch (columnIndex) {
		case 0:
			return quiz;
		case 1:
			return quiz.getLastChangedTime();
		case 2:
			return KreawiQuizHelper.getTitle(quiz);
		case 3:
			return new Integer(quiz.getQuestionCount());
		case 4:
			return KreawiQuizHelper.getProgress(quiz);
		case 5:
			return KreawiQuizHelper.getCorrect(quiz);
		case 6:
			return quiz.getStopwatch().getOverallTimeTakenString();
		case 7:
			return quiz.isExamMode() ? new ImageIcon(TrainerMain.class.getResource(ICON_EXAM_MODE)) : null;
		}

		return TrainerTextResources.getString("HISTORY_TABLE_ERROR");
	}
  
}
