package de.kreawi.trainer.dialogs.onlineGame;

import java.util.Comparator;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.OGButton;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.OnlineGameScrollPane;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.ProfileHeader;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.QuizListView;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.Subheader;
import de.kreawi.trainer.onlinegame.OnlineGameMain;
import de.kreawi.trainer.onlinegame.OnlineGameModel.QuizListChangedListener;
import de.kreawi.trainer.onlinegame.entities.OnlineGameQuiz;
import javafx.application.Platform;
import javafx.scene.control.Button;
import javafx.scene.control.ScrollPane;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;

public class Dashboard extends StackPane {
  

  private final OnlineGameMain onlineGame;

  private final OnlineGameContainer onlineGameContainer;

  private ProfileHeader profileHeader;

  @SuppressWarnings("unused")
  private static final Logger logger = LoggerFactory.getLogger(Dashboard.class); 

  public Dashboard(final OnlineGameMain onlineGame, final OnlineGameContainer onlineGameContainer) {
    
    this.onlineGame = onlineGame;
    this.onlineGameContainer = onlineGameContainer;
    
    initLayout();
    
  }

  private void initLayout() {
    getStylesheets().add("/de/kreawi/trainer/dialogs/kreawi_internal_frame.css");
    
    this.minWidthProperty().bind(this.onlineGameContainer.widthProperty());
    
    final ScrollPane listsPane = new OnlineGameScrollPane();
    onlineGame.getModel().addQuizListChangedListener(new QuizListChangedListener() {
      @Override
      public void listChanged(OnlineGameQuiz newQuiz) {
        Platform.runLater(() -> {
          listsPane.setContent(createQuizListContent());
        });
      }
    });

    listsPane.setContent(createQuizListContent());

    this.profileHeader = new ProfileHeader(this.onlineGame, this.onlineGameContainer);
    
    final BorderPane mainLayer = new BorderPane();
    mainLayer.setTop(this.profileHeader);
    mainLayer.setCenter(listsPane);
    
    
    final Button newGame = new OGButton("+ Neues Spiel", "Ein neues Spiel starten", 
        () -> {
          onlineGameContainer.showFriendsList();
        });
    newGame.layoutYProperty().bind(this.profileHeader.heightProperty().subtract(newGame.heightProperty().divide(2.0)));
    newGame.layoutXProperty().bind(widthProperty().subtract(newGame.widthProperty()).subtract(30));
    final Pane newGameButtonLayer = new Pane(newGame);
    newGameButtonLayer.setPickOnBounds(false);
    
    getChildren().addAll(mainLayer, newGameButtonLayer);
    

    
  }

  private VBox createQuizListContent() {
    final VBox listsPaneContent = new VBox();
    
    Comparator<OnlineGameQuiz> quizComparator = new Comparator<OnlineGameQuiz>() {
      @Override
      public int compare(OnlineGameQuiz ogq1, OnlineGameQuiz ogq2) {
        return (-1) * ogq1.getCreatedTimestamp().compareTo(ogq2.getCreatedTimestamp());
      }
    };
    
    final List<OnlineGameQuiz> currentQuizzesList = onlineGame.getCurrentQuizzes();
    if (!currentQuizzesList.isEmpty()) {
      currentQuizzesList.sort(quizComparator);
      final Subheader header = new Subheader("Läuft gerade");
      final QuizListView currentQuizzes = new QuizListView(currentQuizzesList, this.onlineGameContainer);
      currentQuizzes.minWidthProperty().bind(listsPaneContent.widthProperty());
      listsPaneContent.getChildren().addAll(header, currentQuizzes);
    }
    
    final List<OnlineGameQuiz> finishedQuizzesList = onlineGame.getFinishedQuizzes();
    if (!finishedQuizzesList.isEmpty()) {
      finishedQuizzesList.sort(quizComparator);
      final Subheader header = new Subheader("Beendete Spiele");
      final QuizListView finishedQuizzes = new QuizListView(finishedQuizzesList, this.onlineGameContainer);
      finishedQuizzes.minWidthProperty().bind(listsPaneContent.widthProperty());
      listsPaneContent.getChildren().addAll(header, finishedQuizzes);
    }
    
    listsPaneContent.setFillWidth(true);
    return listsPaneContent;
  }

  public void reloadProfile() {
    this.profileHeader.loadProfile();
  }

}
