package de.kreawi.trainer.dialogs.onlineGame;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import org.codefx.libfx.control.webview.WebViewHyperlinkListener;
import org.codefx.libfx.control.webview.WebViews;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.HeaderWithBackbutton;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.OGButton;
import de.kreawi.trainer.onlinegame.OnlineGameMain;
import de.kreawi.trainer.onlinegame.entities.OnlineGameAnswer;
import de.kreawi.trainer.onlinegame.entities.OnlineGameQuestion;
import de.kreawi.trainer.onlinegame.entities.OnlineGameQuestionResult;
import de.kreawi.trainer.onlinegame.entities.OnlineGameQuiz;
import de.kreawi.trainer.onlinegame.entities.OnlineGameTopic;
import javafx.animation.Animation;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.application.Platform;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonBase;
import javafx.scene.control.CheckBox;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.Toggle;
import javafx.scene.control.ToggleGroup;
import javafx.scene.control.Tooltip;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.Region;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.web.WebView;
import javafx.util.Duration;

public class PlayRound extends BorderPane {

  // Convert Swing colors to JavaFx colors
  private static final Color COLOR_CORRECT = Color.rgb(KreawiGlobals.CORRECT_COLOR.getRed(),
      KreawiGlobals.CORRECT_COLOR.getGreen(), KreawiGlobals.CORRECT_COLOR.getBlue());
  private static final Color COLOR_WRONG = Color.rgb(KreawiGlobals.WRONG_COLOR.getRed(),
      KreawiGlobals.WRONG_COLOR.getGreen(), KreawiGlobals.WRONG_COLOR.getBlue());

  private static final long MAX_QUESTION_TIME = TimeUnit.MINUTES.toMillis(2); 
  private static final DateFormat TIME_FORMAT = new SimpleDateFormat( "m:ss" );

  private final OnlineGameQuiz quiz;
  private OnlineGameTopic topic;
  private final OnlineGameMain onlineGame;
  private final OnlineGameContainer ogc;
  
  private final ToggleGroup radioGroup = new ToggleGroup();
  private final ArrayList<CheckBox> checkboxGroup = new ArrayList<CheckBox>();
  
  private final ArrayList<OnlineGameQuestionResult> results = new ArrayList<OnlineGameQuestionResult>();
  private int currentQuestionIndex = 0;
  private long questionStartTime;
  private long timeTakenTotal = 0l;
  private final HyperlinkListener hyperlinkListener;
  private StackPane commentedAnswerPane;
  
  private Label timeLabel;
  private Timeline questionTimeTakenTimeline;
  
  private static final Logger logger = LoggerFactory.getLogger(PlayRound.class);

  public PlayRound(final OnlineGameQuiz quiz, final OnlineGameTopic topic, final OnlineGameMain onlineGame,
      final OnlineGameContainer ogc, final HyperlinkListener hyperlinkListener) {
    this.quiz = quiz;
    this.topic = topic;
    this.onlineGame = onlineGame;
    this.ogc = ogc;
    this.hyperlinkListener = hyperlinkListener;
    
    getStylesheets().add("/de/kreawi/trainer/dialogs/onlineGame/online_game.css");

    this.currentQuestionIndex = 0;  // show first question
    
    final Runnable backAction = () -> {
      this.ogc.showDashboard();
    };
    
    final HeaderWithBackbutton header = new HeaderWithBackbutton(topic.getName(), backAction);
    this.setTop(header);
    
    showQuestion();
    
  }

  private void showQuestion() {
    
    this.questionStartTime = System.currentTimeMillis();
    
    final Region leftPanes = createLeftPanes();
    final Region rightPanes = createRightPanes();
    final HBox questionLayout = new HBox(leftPanes, rightPanes);
    leftPanes.prefWidthProperty().bind(questionLayout.widthProperty().multiply(.5d));
    rightPanes.prefWidthProperty().bind(questionLayout.widthProperty().multiply(.5d));
    this.setCenter(questionLayout);
  }

  private Region createLeftPanes() {
    final Region questionPane = createQuestionPane();
    questionPane.getStyleClass().add("region");
    questionPane.getStyleClass().add("border-bottom-right");
    final Node answersPane = createAnswersPane();
    answersPane.getStyleClass().add("region");
    answersPane.getStyleClass().add("border-right");
    answersPane.getStyleClass().add("rounded-bottom-left");
    VBox.setVgrow(answersPane, Priority.ALWAYS);
    final VBox vbox = new VBox(questionPane, answersPane);
    questionPane.prefHeightProperty().bind(vbox.heightProperty().multiply(.5d));
    return vbox;
  }

  private Region createQuestionPane() {
    
    final OnlineGameQuestion question = getCurrentQuestion();

    final Region questionWebView = createWebView(question.getBody());
    questionWebView.prefWidthProperty().bind(this.widthProperty().multiply(.33d));
//    final OnlineGameScrollPane questionTextContainer = new OnlineGameScrollPane(questionWebView);
//    questionTextContainer.setVbarPolicy(ScrollBarPolicy.AS_NEEDED);
//    questionTextContainer.prefWidthProperty().bind(this.widthProperty().multiply(.33d));
    return questionWebView;
  }
  
  private Node createAnswersPane() {
    final OnlineGameQuestion question = getCurrentQuestion();
    
    final VBox answersContainer = new VBox(5d);
    final double answersFontSize = 13d;
    this.radioGroup.getToggles().clear();
    this.checkboxGroup.clear();
    final boolean hasMultipleCorrectAnswers = hasMultipleCorrectAnswers(question);
    int answerNo = 0;
    for (OnlineGameAnswer answer : question.getAnswers()) {
      answerNo++;
      final String prefix = (char) (answerNo + 'A' - 1) + ") ";  // prepend letter A) to E)
      final String answerText = prefix + answer.getTitle();
      final ButtonBase answerControl;
      if (hasMultipleCorrectAnswers) {
        answerControl = new CheckBox(answerText);
      } else {
        answerControl = new RadioButton(answerText);
      }
      answerControl.setTooltip(createTooltip(answerText));
      answerControl.setUserData(answer);
      answerControl.setWrapText(true);
      answerControl.setFont(Font.font("Arial", FontWeight.NORMAL, answersFontSize));
      answersContainer.getChildren().add(answerControl);
      if (hasMultipleCorrectAnswers) {
        final CheckBox checkbox = (CheckBox) answerControl;
        checkboxGroup.add(checkbox);
      } else {
        final RadioButton radio = (RadioButton) answerControl;
        radio.setToggleGroup(this.radioGroup);
      }
    }
    
    answersContainer.setAlignment(Pos.CENTER_LEFT);
    answersContainer.setPadding(new Insets(10d, 10d, 10d, 10d));
    
    final VBox centeringVbox = new VBox(answersContainer);
    centeringVbox.setAlignment(Pos.CENTER_LEFT);
    
    return centeringVbox;
  }
  
  private Region createRightPanes() {
    final Node commentedAnswerPane = createCommentedAnswerPane();
    commentedAnswerPane.getStyleClass().add("region");
    commentedAnswerPane.getStyleClass().add("border-bottom");
    BorderPane borderPane = new BorderPane(commentedAnswerPane);
    final Node timeTakenPane = createTimeTakenPane();
    timeTakenPane.getStyleClass().add("region");
    timeTakenPane.getStyleClass().add("border-bottom");
    borderPane.setTop(timeTakenPane);
    final HBox nextButton = createNextButton();
    nextButton.getStyleClass().add("region");
    nextButton.getStyleClass().add("no-border");
    nextButton.getStyleClass().add("rounded-bottom-right");
    borderPane.setBottom(nextButton);
    borderPane.prefWidthProperty().bind(this.widthProperty().multiply(.33d));
//    borderPane.setStyle("-fx-background-color: linear-gradient(to bottom right, #fff, #ddd)");
    return borderPane;
  }

  private Node createTimeTakenPane() {
    this.timeLabel = new Label();
    setTimeTaken(0l);
    final Font font = Font.font("Arial", FontWeight.BOLD, 14d);
    this.timeLabel.setFont(font);
    this.questionTimeTakenTimeline = new Timeline(new KeyFrame(Duration.millis(333), event -> {
      final long diff = System.currentTimeMillis() - PlayRound.this.questionStartTime;
      if (diff >= MAX_QUESTION_TIME) {
        this.timeLabel.setTextFill(Color.RED);
        answerQuestion();
      } else {
        setTimeTaken(diff);
      }
    }));
    questionTimeTakenTimeline.setCycleCount( Animation.INDEFINITE );
    questionTimeTakenTimeline.play();
    HBox hbox = new HBox(timeLabel);
    hbox.setAlignment(Pos.CENTER);
    HBox.setMargin(timeLabel, new Insets(10d));
    return hbox;
  }
  
  private void setTimeTaken(final long millis) {
    this.timeLabel.setText(TIME_FORMAT.format(millis));
  }

  private Region createCommentedAnswerPane() {
    this.commentedAnswerPane = new StackPane();

    final OGButton answerQuestion = new OGButton("Antworten", "Diese Frage beantworten", this::answerQuestion);
    final HBox buttonLayer = new HBox(answerQuestion);
    buttonLayer.setAlignment(Pos.CENTER);
    
    commentedAnswerPane.getChildren().addAll(buttonLayer);
    commentedAnswerPane.prefWidthProperty().bind(this.widthProperty().multiply(.33d));
    return commentedAnswerPane;
  }

  private Region createWebView(final String htmlContent) {
    final WebView webView = new WebView();
    final WebViewHyperlinkListener listener = WebViewHyperlinkListener.fromHyperlinkListener(this.hyperlinkListener,
        true);
    WebViews.addHyperlinkListener(webView, listener, HyperlinkEvent.EventType.ACTIVATED);
    webView.getEngine().loadContent(createHtmlLayout(htmlContent));
    final HBox hbox = new HBox(webView);
    return hbox;
  }
  
  private static String createHtmlLayout(String content) {
    return "<!DOCTYPE html><html><head>"
        + "<meta charset=\"UTF-8\">"
        + "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"/>"
        + "<style>"
        + "a {color: rgb(0,154,205);} "
        + "body {background-color: #eee;" 
        + "      font-family: Arial, Sans-serif; font-weight: normal; font-size: 11pt; "
        + "      color: #222;}"
        + "</style></head><body>"
        + "<div id=\"mydiv\">" + content + "</div>" + "</body></html>";
  }

  private Tooltip createTooltip(final String answerText) {
    final Tooltip tooltip = new Tooltip(answerText);
    tooltip.setFont(Font.font(14d));
    tooltip.setMaxWidth(300d);
    tooltip.setWrapText(true);
    return tooltip;
  }

  private boolean hasMultipleCorrectAnswers(final OnlineGameQuestion question) {
    int count = 0;
    for (OnlineGameAnswer a : question.getAnswers()) {
      if (a.isCorrect()) {
        count++;
      }
    }
    return count > 1;
  }

  private OnlineGameQuestion getCurrentQuestion() {
    return this.topic.getQuestions()[this.currentQuestionIndex];
  }

  private void answerQuestion() {
    final OnlineGameQuestion question = getCurrentQuestion();
    if (isCurrentQuestionAnswered()) {
      logger.warn("Tried to answer question although already answered! Canceling...");
      return;
    }
    final boolean correct;
    if (hasMultipleCorrectAnswers(question)) {
      boolean tmpCorrect = true;
      for (CheckBox c : this.checkboxGroup) {
        final OnlineGameAnswer a = (OnlineGameAnswer) c.getUserData();
        if ((a.isCorrect() && !c.isSelected()) || (!a.isCorrect() && c.isSelected())) {
          tmpCorrect = false;
        }
      }
      correct = tmpCorrect;
    } else {
      if (this.radioGroup.getSelectedToggle() == null) {
        correct = false;
      } else {
        final OnlineGameAnswer answer = (OnlineGameAnswer) this.radioGroup.getSelectedToggle().getUserData();
        correct = answer.isCorrect();
      }
    }
    final String hintsUsed = ""; // unused
    final OnlineGameQuestionResult result = new OnlineGameQuestionResult(question.getId(), correct, hintsUsed);
    this.results.add(result);
    
    Platform.runLater(() -> {
      if (hasMultipleCorrectAnswers(question)) {
        for (CheckBox checkbox : this.checkboxGroup) {
          final OnlineGameAnswer answer = (OnlineGameAnswer) checkbox.getUserData();
          if (answer.isCorrect()) {
            checkbox.getStyleClass().add("correct");
          } else if (checkbox.isSelected()) {
            checkbox.getStyleClass().add("incorrect");
          }
          // setDisable grays out the radio button and the text; so this
          // WORKAROUND:
          checkbox.setMouseTransparent(true);
          checkbox.setFocusTraversable(false);
        }
      } else {
        for (Toggle t : this.radioGroup.getToggles()) {
          RadioButton radio = (RadioButton) t;
          final OnlineGameAnswer answer = (OnlineGameAnswer) radio.getUserData();
          if (answer.isCorrect()) {
            radio.getStyleClass().add("correct");
          } else if (radio.isSelected()) {
            radio.getStyleClass().add("incorrect");
          }
          // setDisable grays out the radio button and the text; so this
          // WORKAROUND:
          radio.setMouseTransparent(true);
          radio.setFocusTraversable(false);
        }
      }
      
      this.questionTimeTakenTimeline.stop();
      final long timeTaken = Math.min(MAX_QUESTION_TIME, System.currentTimeMillis() - this.questionStartTime);
      this.timeTakenTotal += timeTaken;
      setTimeTaken(timeTaken);
      
      final String commentedAnswer = question.getComment();
      final Region commentedAnswerWebView = createWebView(commentedAnswer);
      commentedAnswerWebView.prefWidthProperty().bind(this.widthProperty().multiply(.33d));
      commentedAnswerPane.getChildren().add(commentedAnswerWebView);
    });
    
  }
  
  private HBox createNextButton() {
    
    final String caption;
    final String tooltip;
    final Runnable nextAction;
    
    final int maxIndex = this.topic.getQuestionCount() - 1;
    if (this.currentQuestionIndex < maxIndex) {
      caption = "Weiter";
      tooltip = "nächste Frage anzeigen";
      nextAction = () -> {
        this.currentQuestionIndex += 1;
        showQuestion();
      };
    } else {
      caption = "Runde abschließen";
      tooltip = "Diese Runde abschließen";
      nextAction = () -> {
        submitRound();
      };
    }
    
    final Runnable possiblyAnswerAndNextAction = () -> {
      if (!isCurrentQuestionAnswered()) {
          answerQuestion();
      }
      Platform.runLater(() -> {
        nextAction.run();
      });
    };
    
    final Button next = new OGButton(caption, tooltip, possiblyAnswerAndNextAction);
    final HBox buttonContainer = new HBox(next);
    buttonContainer.setMinHeight(50d);
    buttonContainer.setAlignment(Pos.CENTER);
    return buttonContainer;
  }

  private boolean isCurrentQuestionAnswered() {
    final OnlineGameQuestion currentQuestion = getCurrentQuestion();
    for (OnlineGameQuestionResult result : this.results) {
      if (result.getQuestionId().equals(currentQuestion.getId())) {
        return true;
      }
    }
    return false;
  }

  public void submitRound() {
    
    final int roundNo = this.topic.getRoundNo();
    
    int rightQuestions = 0;
    int wrongQuestions = 0;
    for (OnlineGameQuestionResult result : this.results) {
      if (result.isCorrect()) {
        rightQuestions++;
      } else {
        wrongQuestions++;
      }
    }

    final OnlineGameQuestionResult[] questionResults = 
        this.results.toArray(new OnlineGameQuestionResult[this.results.size()]);
    
    final String timeTaken = Long.valueOf(this.timeTakenTotal).toString();
    
    this.ogc.submitRound(this.quiz, roundNo, timeTaken, rightQuestions,
        wrongQuestions, questionResults);
  }

}
