package de.kreawi.trainer.dialogs.onlineGame;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.HeaderWithBackbutton;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.OGButton;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.OnlineGameScrollPane;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.ProfileImage;
import de.kreawi.trainer.onlinegame.OnlineGameMain;
import de.kreawi.trainer.onlinegame.entities.OnlineGameQuiz;
import de.kreawi.trainer.onlinegame.entities.OnlineGameQuiz.QuizStatus;
import de.kreawi.trainer.onlinegame.entities.OnlineGameResults.OnlineGameResultsPlayer;
import de.kreawi.trainer.onlinegame.entities.OnlineGameResults.OnlineGameResultsQuestion;
import de.kreawi.trainer.onlinegame.entities.OnlineGameResults.OnlineGameResultsRound;
import de.kreawi.trainer.onlinegame.entities.OnlineGameUser;
import de.kreawi.trainer.onlinegame.entities.StatusListener;
import javafx.application.Platform;
import javafx.geometry.HPos;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.geometry.VPos;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.Priority;
import javafx.scene.layout.Region;
import javafx.scene.layout.RowConstraints;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.text.Font;
import javafx.scene.text.FontPosture;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;

public class ResultsOverview extends VBox implements StatusListener {

  private final OnlineGameQuiz quiz;
  private final OnlineGameMain onlineGame;
  private final OnlineGameContainer onlineGameContainer;

  private final OnlineGameScrollPane overview;

  @SuppressWarnings("unused")
  private static final Logger logger = LoggerFactory.getLogger(ResultsOverview.class); 

  public ResultsOverview(final OnlineGameQuiz quiz, final OnlineGameMain onlineGame, final OnlineGameContainer ogc) {
    
    this.quiz = quiz;
    this.onlineGame = onlineGame;
    this.onlineGameContainer = ogc;

    getStylesheets().add("/de/kreawi/trainer/dialogs/onlineGame/online_game.css");
    
    quiz.addStatusListener(this);

    final Runnable backAction = () -> {
      quiz.removeStatusListener(this);
      this.onlineGameContainer.showDashboard();
    };
    getChildren().add(new HeaderWithBackbutton("Spielstand", backAction));
    
    getChildren().add(createResultsHeader());
    
    this.overview = new OnlineGameScrollPane();
    createOverview();
    this.overview.setVvalue(1.0d);
    getChildren().add(overview);
    VBox.setVgrow(overview, Priority.ALWAYS);
  }

  @Override
  public void statusChanged(OnlineGameQuiz quiz, QuizStatus oldStatus, QuizStatus newStatus) {
    final Runnable onSuccess = () -> {
      Platform.runLater(() -> {
        createOverview();
      });
    };
    final Runnable onFailure = () -> {
      logger.warn("Error on renewing quiz results!");
    };
    
    new Thread(() -> {ResultsOverview.this.onlineGame.loadQuizResults(quiz, onSuccess, onFailure);}).start();
  }

  private Node createResultsHeader() {
    
    final double paddingTop = 10d;

    final GridPane gp = new GridPane();
    final ColumnConstraints cc1 = new ColumnConstraints();
    cc1.setHgrow(Priority.ALWAYS);
    cc1.setHalignment(HPos.CENTER);
    cc1.setPercentWidth(45d);
    final ColumnConstraints cc3 = new ColumnConstraints();
    cc3.hgrowProperty().bind(cc1.hgrowProperty());
    cc3.halignmentProperty().bind(cc1.halignmentProperty());
    cc3.percentWidthProperty().bind(cc1.percentWidthProperty());
    final ColumnConstraints cc2 = new ColumnConstraints();
    cc2.setHalignment(HPos.CENTER);
    cc2.setPercentWidth(10d);
    final RowConstraints rc1 = new RowConstraints();
    rc1.setValignment(VPos.CENTER);
    rc1.setFillHeight(false);
    gp.getColumnConstraints().addAll(cc1, cc2, cc3);
    gp.getRowConstraints().add(rc1);
    
    gp.getStyleClass().add("profile-header");
    gp.setPadding(new Insets(paddingTop, 0, 0, 0));
    
    final Text vsLabel = new Text("vs");
    final HBox vsLabelContainer = new HBox(vsLabel);
    vsLabelContainer.setAlignment(Pos.CENTER);
    // correct padding top of GridPane 
    GridPane.setMargin(vsLabelContainer, new Insets(0, 0, paddingTop, 0));
    gp.add(vsLabelContainer, 1, 0, 1, 2);
    
    final OnlineGameUser ownUser = this.onlineGame.getModel().getSettings().getProfileUser();
    gp.addColumn(0, new ProfileImage(ownUser), getUserName(ownUser));

    final OnlineGameUser opponent = this.quiz.getOpponent();
    gp.addColumn(2, new ProfileImage(opponent), getUserName(opponent));

    return gp;
  }

  private static Node getUserName(final OnlineGameUser user) {
    final Text userName = new Text(user.getUserName());
    userName.setTextAlignment(TextAlignment.CENTER);
    userName.getStyleClass().add("header-text");
    final HBox container = new HBox(userName);
    HBox.setMargin(userName, new Insets(5));
    container.setAlignment(Pos.CENTER);
    return container;
  }
  
  private void createOverview() {
    final VBox overview = new VBox();
    overview.setFillWidth(true);
    for (OnlineGameResultsRound round : this.quiz.getResults().getRounds()) {
      overview.getChildren().add(createRoundOverview(round));
    }
    
    if (this.quiz.isFinished()) {
      // Winner name was empty in tests!!!
      // final String winnerName = this.quiz.getResults().getWinnerName();
      if (OnlineGameQuiz.QuizStatus.DRAW.equals(quiz.getStatus())) {
        overview.getChildren().add(getStatusText("Unentschieden!"));
      } else {
        final String winnerName;
        if (this.onlineGame.getModel().getSettings().getProfileUser().getId().equals(this.quiz.getResults().getWinnerId())) {
          winnerName = this.onlineGame.getModel().getSettings().getProfileUser().getUserName();
        } else {
          winnerName = this.quiz.getOpponent().getUserName();
        }
        final String finishedText = winnerName + " hat gewonnen!";
        overview.getChildren().add(getStatusText(finishedText));
      }
    } else if (this.quiz.getStatus() == QuizStatus.ROUND_WAITING) {
      final String waitText = this.quiz.getOpponent().getUserName() + " ist an der Reihe. Sie müssen warten.";
      overview.getChildren().add(getStatusText(waitText));
    } else if (this.quiz.getStatus() == QuizStatus.ROUND_PLAYING) {
      final Runnable nextRound = () -> {
        quiz.removeStatusListener(this);
        onlineGameContainer.playRound(this.quiz);
      };
      final OGButton button = new OGButton("Nächste Runde", "Nächste Runde spielen", nextRound);
      final BorderPane buttonContainer = new BorderPane(button);
      BorderPane.setMargin(button, new Insets(20d));
      overview.getChildren().add(buttonContainer);
    } else if (this.quiz.getStatus() == QuizStatus.SELECT_TOPIC) {
      final Runnable nextRound = () -> {
        quiz.removeStatusListener(this);
        onlineGameContainer.selectTopic(this.quiz);
      };
      final OGButton button = new OGButton("Nächste Runde", "Thema für die nächste Runde wählen", nextRound);
      final BorderPane buttonContainer = new BorderPane(button);
      BorderPane.setMargin(button, new Insets(20d));
      overview.getChildren().add(buttonContainer);
    }
    
    this.overview.setContent(overview);
  }

  private BorderPane getStatusText(final String caption) {
    final Text text = new Text(caption);
    text.setFont(Font.font("Arial, sans-serif", FontPosture.ITALIC, 14d));
    text.setTextAlignment(TextAlignment.CENTER);
    final BorderPane borderPane = new BorderPane(text);
    final double padding = 20d;
    BorderPane.setMargin(text, new Insets(padding));
    text.wrappingWidthProperty().bind(borderPane.widthProperty().subtract(padding * 2d));
    return borderPane;
  }

  private Node createRoundOverview(final OnlineGameResultsRound round) {
    
    Region ownResults = null;
    Region opponentResults = null;
    for (OnlineGameResultsPlayer player : round.getPlayers()) {
      final Region results;
      final String userStatus = player.getUserStatus();
      if ("FINISHED".equals(userStatus) || "SELECT_TOPIC".equals(userStatus)) {
        // These are the status where questions are already answered
        // It seems impossible to find this out in another way
        // because getQuestions is always filled with 6 questions
        // (if unanswered the questions correct flag is false)
        results = createUserResults(player.getQuestions());
      } else {
        final Text isPlayingText = new Text("Spielt gerade");
        isPlayingText.setFont(Font.font("Arial, sans-serif", FontWeight.BOLD, 11d));
        isPlayingText.setTextAlignment(TextAlignment.CENTER);
        final HBox hbox = new HBox(isPlayingText);
        hbox.setAlignment(Pos.CENTER);
        // fix width and height of isPlayingText container
        hbox.setMinWidth(46d);
        hbox.setMaxWidth(46d);
        hbox.setMinHeight(30d);
        isPlayingText.wrappingWidthProperty().bind(hbox.widthProperty());
        results = hbox;
      }
      if (player.getUserId().equals(this.onlineGame.getModel().getSettings().getProfileUser().getId())) {
        ownResults = results;
      } else {
        opponentResults = results;
      }
    }
    if (ownResults == null || opponentResults == null) {
      logger.error("Could not create results");
      return new Pane();
    }
    
    final Label roundLabel = new Label("Runde " + round.getRoundNo());
    roundLabel.getStyleClass().add("round-label");
    
    final Text topicName = new Text(round.getTopicName());
    topicName.setTextAlignment(TextAlignment.CENTER);
    final double topicNamePadding = 3d;
    final BorderPane topicNameContainer = new BorderPane(topicName);
    BorderPane.setMargin(topicName, new Insets(topicNamePadding));
    topicNameContainer.translateYProperty().bind(roundLabel.heightProperty().divide(4d));
    topicName.wrappingWidthProperty().bind(topicNameContainer.widthProperty().subtract(topicNamePadding * 2d));
    topicName.setFont(Font.font("Arial, sans-serif", 10d));
    
    final BorderPane resultsContainer = new BorderPane();
    resultsContainer.setLeft(ownResults);
    resultsContainer.setCenter(topicNameContainer);
    resultsContainer.setRight(opponentResults);
    resultsContainer.setStyle("-fx-border-color: #bbd;-fx-border-width: 1px;-fx-border-style:solid");
    BorderPane.setMargin(ownResults, new Insets(5d));
    BorderPane.setMargin(opponentResults, new Insets(5d));
    
    StackPane.setAlignment(roundLabel, Pos.TOP_CENTER);
    StackPane.setMargin(roundLabel, new Insets(1d, 0, 0, 0));

    final double marginResultsContainer = 2d;
    StackPane.setMargin(resultsContainer, new Insets(12d, marginResultsContainer, 0, marginResultsContainer));
    StackPane.setAlignment(resultsContainer, Pos.BOTTOM_CENTER);
    
    final StackPane pane = new StackPane(resultsContainer, roundLabel);
    resultsContainer.minWidthProperty().bind(pane.widthProperty().subtract(marginResultsContainer * 2d));
    
    return pane;
  }

  private GridPane createUserResults(final OnlineGameResultsQuestion[] questions) {
    final GridPane gp = new GridPane();
    gp.setHgap(2d);
    gp.setVgap(2d);
    
    int i = 0;
    for (OnlineGameResultsQuestion question : questions) {
      final Node child = new Circle(7, question.isCorrect()?Color.GREEN:Color.RED);
      int columnIndex = i % 3;
      int rowIndex = i / 3;
      gp.add(child, columnIndex, rowIndex);
      i++;
    }
    
    return gp;
  }
  
}
