package de.kreawi.trainer.dialogs.onlineGame;

import java.util.List;

import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.HeaderWithBackbutton;
import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.TopicSelector;
import de.kreawi.trainer.onlinegame.entities.OnlineGameQuiz;
import de.kreawi.trainer.onlinegame.entities.OnlineGameTopic;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;

public class SelectTopic extends BorderPane {
  
  private final OnlineGameQuiz quiz;
  private final List<OnlineGameTopic> topics;
  private final OnlineGameContainer onlineGameContainer;
  
  public SelectTopic(final OnlineGameQuiz quiz, final List<OnlineGameTopic> topics,
      final OnlineGameContainer onlineGameContainer) {
    this.quiz = quiz;
    this.topics = topics;
    this.onlineGameContainer = onlineGameContainer;
    
    final Runnable backAction = () -> {
      this.onlineGameContainer.showDashboard();
    };
    final HeaderWithBackbutton header = new HeaderWithBackbutton("Bitte wählen Sie ein Thema", backAction);
    setTop(header);
    
    
    final Pane topicsPane = createTopicsPane();
    topicsPane.maxWidthProperty().bind(maxWidthProperty());
    setCenter(topicsPane);
    
    final Text notesHeader = new Text("Hinweise:");
    final HBox notesHeaderBox = new HBox(notesHeader);
    notesHeaderBox.setAlignment(Pos.CENTER);
    final Text note1 = new Text("- Spieldauer: 4 Runden je 6 Fragen.");
    final Text note2 = new Text("- Wenn ein/e Spieler/in die Runde innerhalb von 48 Stunden nicht " + 
                                "abgeschlossen hat, dann verliert er/sie das Spiel.");
    double margin = 9d;
    note2.wrappingWidthProperty().bind(widthProperty().subtract(margin * 2d));
    final VBox notes = new VBox(5, notesHeaderBox, note1, note2);
    setBottom(notes);
    BorderPane.setMargin(notes, new Insets(margin));
  }
  
  private Pane createTopicsPane() {
    
    final OnlineGameTopic topic1 = this.topics.get(0);
    final TopicSelector circle1 = new TopicSelector(topic1.getName(), Color.rgb(238,92,66), getTopicSelectedAction(topic1));
    
    final OnlineGameTopic topic2 = this.topics.get(1);
    final TopicSelector circle2 = new TopicSelector(topic2.getName(), Color.rgb(0,205,102), getTopicSelectedAction(topic2));

    final OnlineGameTopic topic3 = this.topics.get(2);
    final TopicSelector circle3 = new TopicSelector(topic3.getName(), Color.rgb(0,154,205), getTopicSelectedAction(topic3));
    
    final Pane topicsContainer = new Pane(circle1, circle2, circle3);
    topicsContainer.maxWidthProperty().bind(widthProperty());
    final double circleSize = 0.45;   // circle size relative to container width
    final double lowerCirclesDistance = 7d;
    circle1.prefWidthProperty().bind(topicsContainer.widthProperty().multiply(circleSize));
    circle1.prefHeightProperty().bind(circle1.prefWidthProperty());
    circle1.maxWidthProperty().bind(topicsContainer.widthProperty().multiply(circleSize));
    circle1.maxHeightProperty().bind(circle1.maxWidthProperty());
    circle1.layoutYProperty().bind(topicsContainer.heightProperty().divide(2d).subtract(circle1.prefHeightProperty()));
    circle1.layoutXProperty().bind(topicsContainer.widthProperty().divide(2d).subtract(circle1.widthProperty().divide(2d)));
    circle2.prefWidthProperty().bind(topicsContainer.widthProperty().multiply(circleSize));
    circle2.prefHeightProperty().bind(circle2.prefWidthProperty());
    circle2.maxWidthProperty().bind(topicsContainer.widthProperty().multiply(circleSize));
    circle2.maxHeightProperty().bind(circle2.maxWidthProperty());
    circle2.layoutXProperty().bind(topicsContainer.widthProperty().divide(2d).subtract(circle2.widthProperty()).subtract(lowerCirclesDistance));
    circle2.layoutYProperty().bind(topicsContainer.heightProperty().divide(2d));
    circle3.prefWidthProperty().bind(topicsContainer.widthProperty().multiply(circleSize));
    circle3.prefHeightProperty().bind(circle3.prefWidthProperty());
    circle3.maxWidthProperty().bind(topicsContainer.widthProperty().multiply(circleSize));
    circle3.maxHeightProperty().bind(circle3.maxWidthProperty());
    circle3.layoutXProperty().bind(topicsContainer.widthProperty().divide(2d).add(lowerCirclesDistance));
    circle3.layoutYProperty().bind(topicsContainer.heightProperty().divide(2d));
    
    return topicsContainer;
  }

  private Runnable getTopicSelectedAction(final OnlineGameTopic topic) {
    final Runnable action1 = () -> {
      this.onlineGameContainer.playRound(this.quiz, topic);
    };
    return action1;
  }

}
