package de.kreawi.trainer.dialogs.onlineGame.custom_nodes;

import java.lang.reflect.Field;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;

import com.sun.webkit.WebPage;

import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.ListChangeListener;
import javafx.concurrent.Worker.State;
import javafx.event.EventHandler;
import javafx.geometry.HPos;
import javafx.geometry.VPos;
import javafx.scene.Node;
import javafx.scene.input.ScrollEvent;
import javafx.scene.layout.Pane;
import javafx.scene.layout.StackPane;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;
import netscape.javascript.JSException;

public final class WebViewFitContent extends StackPane {
  
  final WebView webview = new WebView();
  final WebEngine webEngine = webview.getEngine();
  
  private static final Logger logger = LoggerFactory.getLogger(WebViewFitContent.class);

  public WebViewFitContent(String content) {
    
    webEngine.documentProperty().addListener(new WebDocumentListener(webEngine));
    
    webview.setPrefHeight(5);
    
    widthProperty().addListener(new ChangeListener<Object>() {
      @Override
      public void changed(ObservableValue<?> observable, Object oldValue, Object newValue) {
        Double width = (Double) newValue;
        webview.setPrefWidth(width);
        adjustHeight();
      }
    });
    
    webview.getEngine().getLoadWorker().stateProperty().addListener(new ChangeListener<State>() {
      @Override
      public void changed(ObservableValue<? extends State> arg0, State oldState, State newState) {
        if (newState == State.SUCCEEDED) {
          adjustHeight();
          disableScrolling();
        }
      }
    });
    
    webview.getChildrenUnmodifiable().addListener(new ListChangeListener<Node>() {
      @Override
      public void onChanged(ListChangeListener.Change<? extends Node> change) {
        Set<Node> scrolls = webview.lookupAll(".scroll-bar");
        for (Node scroll : scrolls) {
          scroll.setVisible(false);
        }
      }
    });
    
    setContent(content);
    getChildren().add(webview);
    
    // prevent mouse wheel scrolling (produces buggy output on webview)
//    webview.setOnScroll(new EventHandler<ScrollEvent>() {
//      @Override
//      public void handle(ScrollEvent event) {
//        event.consume();
//        logger.info("Scrolling in webview consumed!");
//      }
//    });
//    webview.addEventFilter(ScrollEvent.ANY, new EventHandler<ScrollEvent>() {
//      @Override
//      public void handle(ScrollEvent event) {
//          event.consume();
//      }
//    });
  }
  
  public void setContent(final String content) {
    Platform.runLater(new Runnable() {
      @Override
      public void run() {
        webEngine.loadContent(getHtml(content));
        Platform.runLater(new Runnable() {
          @Override
          public void run() {
            adjustHeight();
          }
        });
      }
    });
  }
  
  @Override
  protected void layoutChildren() {
    double w = getWidth();
    double h = getHeight();
    layoutInArea(webview, 0, 0, w, h, 0, HPos.CENTER, VPos.CENTER);
  }
  
  private void adjustHeight() {
    Platform.runLater(new Runnable() {
      @Override
      public void run() {
        try {
          Object result = webEngine
              .executeScript("var myDiv = document.getElementById('mydiv');" + "if (myDiv != null) myDiv.offsetHeight");
          if (result instanceof Integer) {
            Integer i = (Integer) result;
            double height = new Double(i);
            height = height + 30;
            webview.setPrefHeight(height);
          }
        } catch (JSException e) {
          e.printStackTrace();
        }
      }
    });
  }
  
  private void disableScrolling() {
    
    final String disableScrollingJs = "var styleElement = document.createElement('style');"
        + "var styleText = 'body, html, div { overflow: hidden; }';"
        + "var headElements = document.getElementsByTagName('head');" + "styleElement.type = 'text/css';"
        + "if (headElements.length == 1) {"
        + "  headElements[0].appendChild(styleElement);"
        + "} else if (document.head) {"
        + "  document.head.appendChild(styleElement);" 
        + "}"
        + "if (styleElement.styleSheet) {"
        + "  styleElement.styleSheet.cssText = styleText;"
        + "}";
    webEngine.executeScript(disableScrollingJs);
  }
  
  private String getHtml(String content) {
    return "<html><body style=\"background-color: transparent; "
        + "font-family: Arial, Sans-serif; font-weight: bold; font-size: 11pt; color: #222 \">"
        + "<div id=\"mydiv\">" + content + "</div>" + "</body></html>";
  }
  
  public static class WebDocumentListener implements ChangeListener<Document> {
    private final WebEngine webEngine;
    
    public WebDocumentListener(WebEngine webEngine) {
      this.webEngine = webEngine;
    }
    
    @Override
    public void changed(ObservableValue<? extends Document> arg0, Document arg1, Document arg2) {
      try {
        // FIXME: This only works with special "add exports" hack on compilation / execution
        // Use reflection to retrieve the WebEngine's private 'page' field.
        Field f = webEngine.getClass().getDeclaredField("page");
        f.setAccessible(true);
        WebPage page = (WebPage) f.get(webEngine);
        // Set the background color of the page to be transparent.
        page.setBackgroundColor((new java.awt.Color(1.0f, 1.0f, 1.0f, 0.0f)).getRGB());
      } catch (Exception e) {
        System.out.println("Error: " + e);
      }
    }
  }
  
}