package de.kreawi.trainer.dialogs.quiz;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JToggleButton;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jquizengine.questions.Question;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.util.TrainerHtmlUtil;

@SuppressWarnings("serial")
public class MultipleChoicesPanel extends JPanel implements Scrollable {

	private int maxUnitIncrement = 10;

	List<Item> items = new ArrayList<Item>();

	private Font font;

	private boolean doLineWrap;

	private static final Logger logger = LoggerFactory.getLogger(MultipleChoicesPanel.class); 

	public MultipleChoicesPanel(List<String> choices, Question.Type type,
			Font font, boolean doLineWrap) {
		super();
		
		this.font = font;
		this.doLineWrap = doLineWrap;

		setOpaque(false);
		setLayout(new GridLayout(1, 1));

		ButtonGroup group = new ButtonGroup();
		JToggleButton toggle;

		JPanel panel = new JPanel(new GridBagLayout());
		panel.setOpaque(false);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0, 0, 0, 0);
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.weightx = 1;
		gbc.weighty = 4;

		JPanel gluePanel = new JPanel();
		gluePanel.setOpaque(false);
		panel.add(gluePanel, gbc);

		for (int index = 0; index < choices.size(); index++) {

			String choice = choices.get(index);

			if (type == Question.Type.COMBI || type == Question.Type.SINGLE) {
				toggle = new JRadioButton(new Choice("", choice));
				group.add(toggle);
			} else {
				toggle = new JCheckBox(new Choice("", choice));
			}

			JPanel innerPanel = new JPanel(new GridBagLayout());
			innerPanel.setOpaque(false);

			JLabel indexLabel = new JLabel((char) (65 + index)
					+ ") ");
			
			indexLabel.setFont(font);
			
			final JToggleButton tempToggle = toggle;
			MouseAdapter mouseAdapter = new MouseAdapter() {

				@Override
				public void mousePressed(MouseEvent e) {
					tempToggle.doClick();
				}

				// TODO mousePressed is used in an attempt to improve user
				// experience, as sometimes mouse clicks didn't register when
				// the mouse was moved away
//				public void mouseClicked(MouseEvent e) {
//					tempToggle.doClick();
//				}

				public void mouseEntered(MouseEvent e) {
					e.setSource(tempToggle);
					for (MouseListener ml : tempToggle.getMouseListeners()) {
						ml.mouseEntered(e);
					}
				}

				public void mouseExited(MouseEvent e) {
					e.setSource(tempToggle);
					for (MouseListener ml : tempToggle.getMouseListeners()) {
						ml.mouseExited(e);
					}
				}
			};
			
			indexLabel.addMouseListener(mouseAdapter);
			
			toggle.setBorder(BorderFactory.createEmptyBorder());
			
			int topInsetToggle = Math.max(0, Math.round((indexLabel.getFontMetrics(
					indexLabel.getFont()).getHeight() - toggle
					.getPreferredSize().height) / 2f));
			
			gbc = new GridBagConstraints();
			gbc.fill = GridBagConstraints.NONE;
			gbc.insets = new Insets(topInsetToggle, 0, 0, 0);
			gbc.gridy = 0;
			gbc.gridx = 0;
			gbc.weightx = 0;
			gbc.weighty = 0;
			gbc.anchor = GridBagConstraints.NORTHWEST;

			innerPanel.add(toggle, gbc);
			
			int topInsetLabel = Math.max(0, Math.round((toggle
					.getPreferredSize().height - indexLabel.getFontMetrics(
					indexLabel.getFont()).getHeight()) / 2f));

			gbc = new GridBagConstraints();
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.insets = new Insets(topInsetLabel, toggle.getIconTextGap(), 0, 0);
			gbc.gridy = 0;
			gbc.gridx = 1;
			gbc.weightx = 0;
			gbc.weighty = 0;
			gbc.anchor = GridBagConstraints.NORTHWEST;

			innerPanel.add(indexLabel, gbc);

			JLabel label = new JLabel();
			
			label.setFont(this.font);

			label.setText(wrapInHtml(choice, label.getForeground()));
			
			if (KreawiGlobals.isDevMode()) {
				label.setToolTipText(choice);
			}

			label.addMouseListener(mouseAdapter);

			gbc = new GridBagConstraints();
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.insets = new Insets(topInsetLabel, 0, 0, 0);
			gbc.gridy = 0;
			gbc.gridx = 2;
			gbc.weightx = 1;
			gbc.weighty = 1;
			gbc.anchor = GridBagConstraints.NORTHWEST;

			innerPanel.add(label, gbc);

			items.add(new Item(toggle, new ChoiceLabels(indexLabel, label)));

			gbc = new GridBagConstraints();
			gbc.fill = GridBagConstraints.BOTH;
			gbc.insets = new Insets(0, 30, 1, 10);
			gbc.gridy = index + 1;
			gbc.gridx = 0;
			gbc.gridheight = 1;
			gbc.gridwidth = 1;
			gbc.weightx = 1;
			gbc.weighty = 8;

			toggle.setOpaque(false);
			toggle.setFont(this.font);
			toggle.setFocusPainted(false);

			panel.add(innerPanel, gbc);

		}

		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(0, 0, 0, 0);
		gbc.gridy = choices.size() + 1;
		gbc.gridx = 0;
		gbc.gridheight = 1;
		gbc.gridwidth = 1;
		gbc.weightx = 1;
		gbc.weighty = 0;

		gluePanel = new JPanel();
		gluePanel.setOpaque(false);
		panel.add(gluePanel, gbc);
		
		add(panel);

	}

	private String wrapInHtml(String text, Color fg) {
    return "<html><body text=\"" + TrainerHtmlUtil.getHTMLColorString(fg) + "\">" + text + "</body></html>";
	}

	public int getSelected() {
		int ret = 0;
		for (int i = 0; i < this.items.size(); i++) {
			if (this.items.get(i).toggle.isSelected()) {
				ret |= indexToBits(i);
			}
		}
		return ret;
	}

	public void setSelected(int selectedBits) {

		if (selectedBits < 0) {
			return;
		}

		for (int i = 0; i < this.items.size(); i++) {
			if (bitsContainIndex(selectedBits, i)) {
				this.items.get(i).toggle.setSelected(true);
			}
		}

		revalidate();
		repaint();

	}

	public void highlightCorrectAnswers(int correctAnswerBits) {

		for (int i = 0; i < this.items.size(); i++) {

			Item item = this.items.get(i);

			String choiceText = (String) item.toggle.getAction().getValue(
					Choice.CHOICE_TEXT);

			if (KreawiGlobals.isDevMode()) {
			  logger.info("correctAnswerBits: " + correctAnswerBits);
			  logger.info("i + 1: " + (i + 1));
        logger.info("bitsContainIndex: " + bitsContainIndex(correctAnswerBits, i));
			}

			if (bitsContainIndex(correctAnswerBits, i)) {

				Color color = item.toggle.isSelected() ? KreawiGlobals.CORRECT_COLOR
						: KreawiGlobals.WRONG_COLOR;

				item.labels.textLabel.setText(wrapInHtml(choiceText, color));
				item.labels.indexLabel.setForeground(color);
			} else {
				Color color = UIManager.getColor("Label.disabledForeground");
				item.labels.textLabel.setText(wrapInHtml(choiceText, color));
				item.labels.indexLabel.setForeground(color);
			}
		}
		revalidate();
		repaint();
	}
	
	private boolean bitsContainIndex(int bits, int index) {
		return (bits & indexToBits(index)) != 0;
	}
	
	private int indexToBits(int index) {
		return (int)Math.pow(2, index);
	}

	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);

		for (Item item : this.items) {
			item.toggle.setEnabled(enabled);
		}
	}

	private static class Choice extends AbstractAction {

		static final String BULLET_STRING = "bulletString";

		static final String CHOICE_TEXT = "choiceText";

		Choice(String bulletString, String choiceText) {
			putValue(BULLET_STRING, bulletString);
			putValue(CHOICE_TEXT, choiceText);
			putValue(Action.NAME, bulletString);
		}

		@SuppressWarnings("unused")
		String getBulletString() {
			return (String) getValue(Choice.BULLET_STRING);
		}

		@SuppressWarnings("unused")
		String getChoiceText() {
			return (String) getValue(Choice.CHOICE_TEXT);
		}

		public String toString() {
			return (String) getValue(Action.NAME);
		}

		public void actionPerformed(ActionEvent e) {
			/* do nothing */
		}
	}
	
	private static class ChoiceLabels {
		JLabel indexLabel;
		JLabel textLabel;

		ChoiceLabels(JLabel indexLabel, JLabel textLabel) {
			this.indexLabel = indexLabel;
			this.textLabel = textLabel;
		}
	}

	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}

	public boolean getScrollableTracksViewportHeight() {
		/*
		 * return true, if the visible rectangle size is 0 (panel may already be
		 * visible, but layout is not finished; this avoids a visible layout
		 * "jumping") OR if the visible rectangle is bigger than the minimum
		 * needed size of this panel (all fits, no scroll bar needed)
		 */
		return !(getVisibleRect().getSize().height > 0)
				|| getVisibleRect().getSize().height >= getMinimumSize().height;
	}

	public boolean getScrollableTracksViewportWidth() {
		return doLineWrap;
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect,
			int orientation, int direction) {

		return maxUnitIncrement;
	}

	public int getScrollableBlockIncrement(Rectangle visibleRect,
			int orientation, int direction) {
		if (orientation == SwingConstants.HORIZONTAL) {
			return visibleRect.width - maxUnitIncrement;
		} else {
			return visibleRect.height - maxUnitIncrement;
		}
	}
	
	public void addActionListener(ActionListener listener) {
		for (Item item : this.items) {
			item.toggle.addActionListener(listener);
		}
	}
	
	public void removeActionListener(ActionListener listener) {
		for (Item item : this.items) {
			item.toggle.removeActionListener(listener);
		}
	}
	
	private static class Item {
		JToggleButton toggle;
		ChoiceLabels labels;

		Item(JToggleButton toggle, ChoiceLabels labels) {
			this.toggle = toggle;
			this.labels = labels;
		}
	}
}
