package de.kreawi.trainer.dialogs.quiz;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

import javax.swing.JLabel;
import javax.swing.Timer;

import com.jquizengine.quiz.Quiz;


public class QuizTitleClock extends Timer {

	private static final long serialVersionUID = -3202515507469040832L;

	private ClockActionListener clockActionListener;

	public QuizTitleClock(int delay, Quiz model, JLabel label,
			String prefix, String postfix, long questionTimeLimit,
			boolean doCountdown) {
		super(delay, null);

		this.clockActionListener = new ClockActionListener(model, label,
				prefix, postfix, questionTimeLimit, doCountdown);
		addActionListener(this.clockActionListener);
	}

	private static class ClockActionListener implements ActionListener {

		private Quiz model;

		private long questionTimeLimit;

		private boolean doCountdown;

		private boolean hideTime;

		private String time;

		private HashMap<JLabel, ClockStringTuple> clockStringMap = new HashMap<JLabel, ClockStringTuple>();

		public ClockActionListener(Quiz model, JLabel label,
				String prefix, String postfix, long questionTimeLimit,
				boolean doCountdown) {

			this.model = model;
			this.questionTimeLimit = questionTimeLimit;
			this.doCountdown = doCountdown;
			this.clockStringMap.put(label, new ClockStringTuple(prefix, postfix));

		}

		public void actionPerformed(ActionEvent e) {
			performClockUpdate();
		}

		public boolean isHidingTime() {
			return hideTime;
		}

		public void performClockUpdate() {
			hideTime = !hideTime;

			if (model.getStopwatch().isRunning()
					&& model.isQuestionsTimeLimitExceeded(questionTimeLimit,
							true) && hideTime) {
				time = "";
			} else {
				if (this.doCountdown) {
					time = model
							.getStopwatch()
							.getOverallTimeTakenString(
									model.getQuizTimeLimit(questionTimeLimit) + 999);
					/*
					 * +999 for cosmetic reasons, otherwise countdown starts
					 * always at countdownFrom - 1 second
					 */
				} else {
					time = model.getStopwatch()
							.getOverallTimeTakenString();
				}
			}

			for (JLabel label : clockStringMap.keySet()) {
				String oldText = label.getText();
				String newText = clockStringMap.get(label).prefix + time
						+ clockStringMap.get(label).postfix;
				Dimension oldPreferredSize = label.getPreferredSize();
				if (oldText.length() > newText.length()) {
					label.setMinimumSize(oldPreferredSize);
				}

				label.setText(newText);
			}
		}
		
		public void addLabel(JLabel label, String prefix, String postfix) {
			this.clockStringMap.put(label,
					new ClockStringTuple(prefix, postfix));
		}
		
	}

	@Override
	public void stop() {
		super.stop();

		if (this.clockActionListener.isHidingTime()) {
			this.clockActionListener.performClockUpdate();
		}
	}
	
	public void addLabel(JLabel label, String prefix, String postfix) {
		this.clockActionListener.addLabel(label, prefix, postfix);
	}
	
	private static class ClockStringTuple {
		String prefix;
		String postfix;
		
		ClockStringTuple(String prefix, String postfix) {
			this.prefix = prefix;
			this.postfix = postfix;
		}
	}

}
