package de.kreawi.trainer.license;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;

public class LicenseFileHandler {
  
  private static Properties s_keyProperties;
  protected static File s_keyFile;
  
  private static final String LICENSE_FILE_NAME = "licence.xml";

  public static final String KEY_FAILED_COUNT = "lusefrjh" + KreawiGlobals.CURRENT_YEAR;
  public static final String KEY_FIRST_START = "AAAAAAA" + KreawiGlobals.CURRENT_YEAR;
  
  // For registration
  public static final String KEY_REG_CLIENT_UID = "XYZ" + KreawiGlobals.CURRENT_YEAR;
  public static final String KEY_REG_EMAIL_ADDRESS = "email" + KreawiGlobals.CURRENT_YEAR;
  public static final String KEY_REG_COMPUTER_NAME = "computerName" + KreawiGlobals.CURRENT_YEAR;
  public static final String KEY_REG_LAST_SERVER_STATUS = "ggg" + KreawiGlobals.CURRENT_YEAR;
  public static final String KEY_REG_LAST_SERVER_STATUS_REASON = "rrr" + KreawiGlobals.CURRENT_YEAR;
  
  private static final Logger logger = LoggerFactory.getLogger(LicenseFileHandler.class); 

  private LicenseFileHandler() {
    // no instantiation
  }
  
  private static String getLicenseKeyPropertyKey() {
    return "licenceKey";
  }
  
  public static String getCurrentLicenceKey() {
    return getProperty(getLicenseKeyPropertyKey());
  }
  
  public static String getLicenseKey() {
    return getProperty(getLicenseKeyPropertyKey());
  }
  
  public static void setLicenseKey(final String enteredKey) {
    setProperty(getLicenseKeyPropertyKey(), enteredKey);
  }
  
  public static void possiblySetFirstStart() {
    if (!containsKey(KEY_FIRST_START)) {
      setProperty(KEY_FIRST_START, Long.toHexString(System.currentTimeMillis()));
    }
  }
  
  public static long getFirstStart() {
    final String firstStart = getProperty(KEY_FIRST_START);
    return (firstStart == null?0:Long.parseLong(firstStart, 16));
  }
  
  public static void setRegistration(String uid, String email, String computerName, int serverStatus) {
    setProperty(KEY_REG_CLIENT_UID, uid);
    setProperty(KEY_REG_EMAIL_ADDRESS, email);
    setProperty(KEY_REG_COMPUTER_NAME, computerName);
    setProperty(KEY_REG_LAST_SERVER_STATUS, "" + serverStatus);
  }
  
  public static String getRegUid() {
    return getProperty(KEY_REG_CLIENT_UID);
  }

  public static String getRegEmail() {
    return getProperty(KEY_REG_EMAIL_ADDRESS);
  }

  public static String getRegComputerName() {
    return getProperty(KEY_REG_COMPUTER_NAME);
  }

  public static void setRegLastServerStatus(final int status, final String reason) {
    setProperty(KEY_REG_LAST_SERVER_STATUS, "" + status);
    setProperty(KEY_REG_LAST_SERVER_STATUS_REASON, reason);
  }
  
  public static int getRegLastServerStatus() {
    final String property = getProperty(KEY_REG_LAST_SERVER_STATUS);
    if (property != null) {
      return Integer.parseInt(property);
    } else {
      return -1;
    }
  }

  public static String getRegLastServerStatusReason() {
    final String property = getProperty(KEY_REG_LAST_SERVER_STATUS_REASON);
    if (property != null) {
      return property;
    } else {
      return "";
    }
  }
  
  public static void setFailedCount(final String failedCount) {
    setProperty(LicenseFileHandler.KEY_FAILED_COUNT, failedCount);
  }
  
  public static String getFailedCount() {
    return getProperty(LicenseFileHandler.KEY_FAILED_COUNT);
  }

  private static boolean containsKey(final String key) {
    return getProperties().containsKey(key);
  }
  
  private static void setProperty(final String key, final String value) {
    if (value == null) {
      removeProperty(key);
    } else {
      getProperties().setProperty(key, value);
      storeProperties();
    }
  }
  
  private static String getProperty(final String key) {
    return getProperties().getProperty(key);
  }
  
  private static void removeProperty(final String key) {
    getProperties().remove(key);
    storeProperties();
  }
  
  private static Properties getProperties() {
    if (s_keyProperties == null) {
      s_keyProperties = new Properties();
      s_keyFile = new File(KreawiGlobals.getApplicationDirectory(), LICENSE_FILE_NAME);

      try {
        s_keyProperties.loadFromXML(new FileInputStream(s_keyFile));
        logger.info("Successfully read license file (" + s_keyFile + ")");
      } catch (InvalidPropertiesFormatException e) {
        logger.error("Error while loading license file!", e);
      } catch (FileNotFoundException e) {
        logger.info("License file not found. Creating new one!");
      } catch (IOException e) {
        logger.error("Error while loading license file!", e);
      }
    }
    return s_keyProperties;
  }
  
  private static void storeProperties() {
    try {
      getProperties().storeToXML(new FileOutputStream(s_keyFile), null);
    } catch (FileNotFoundException e) {
      logger.error("lklflclnlblwl!", e);
    } catch (IOException e) {
      logger.error("lklflclnlblwl!", e);
    }
  }
  
  public static void resetProperties() {
    s_keyProperties.clear();
    s_keyFile.delete();
  }
  
}
