package de.kreawi.trainer.quiz;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jquizengine.questions.Question;
import com.jquizengine.questions.collection.QuestionPool;
import com.jquizengine.quiz.Quiz;
import com.jquizengine.quiz.Response;
import com.jquizengine.quiz.Stopwatch;

import de.kreawi.trainer.dialogs.quiz.QuizTimeTracker;
import de.martinknauer.util.Util;

public class QuizModel implements Serializable {

	private static final long serialVersionUID = -8594942341685260550L;

	private boolean hasOnlyTextQuestions;

	private ArrayList<Integer> ids = new ArrayList<Integer>();

	private int index;

	private Map<Integer, QuestionReply> replies = new HashMap<Integer, QuestionReply>();

	private boolean isExamMode;

	private QuizTimeTracker quizTimeTracker;

	private long startTime = -1;

	private long lastChangedTime = -1;

	private static final Logger logger = LoggerFactory.getLogger(QuizModel.class); 

	public static KreawiQuiz loadQuizFromFile(File file, QuestionPool pool)
			throws IOException, ClassNotFoundException, ClassCastException,
			CloneNotSupportedException {
		Object obj = Util.readObjectFromFile(file);

		KreawiQuiz quiz;

//		if (obj instanceof QuizModel) {
//			QuizModel quizModel = (QuizModel) obj;
////			quiz = Quiz.createFromQuiz(new QuizAdapter(quizModel, pool));
//			quiz = new QuizAdapter(quizModel, pool);
//		} else {
			quiz = (KreawiQuiz) obj;
			quiz.propUpWithQuestionPool(pool);
//		}

		return quiz;
	}

	private static class QuizAdapter extends Quiz {

		private static final long serialVersionUID = 3702207687540580891L;

		public QuizAdapter(QuizModel quizModel, QuestionPool pool) {

			this.ids.addAll(quizModel.ids);

			for (Question question : pool.getQuestions()) {
				if (quizModel.replies.containsKey(question.getId())) {
					QuestionReply reply = quizModel.replies.get(question
							.getId());
					this.responses.put(
							question.getId(),
							new Response(question, reply.getReply(), reply
									.getTimeTaken(), reply.isCommitted()));
				}
			}
			
			propUpWithQuestionPool(pool);
			
			this.hasOnlyTextQuestions = quizModel.hasOnlyTextQuestions;
			this.isExamMode = quizModel.isExamMode;
			this.startTime = quizModel.startTime;
			this.lastChangedTime = quizModel.lastChangedTime;
			this.index = quizModel.index;

			this.stopwatch = new Stopwatch(
					quizModel.quizTimeTracker.getQuestionTime(),
					quizModel.quizTimeTracker.getOverallQuestionDuration());

		}
	}

}
