package de.kreawi.trainer.update;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.KreawiGlobals;
import de.martinknauer.util.Util;
import de.martinknauer.util.progress.ProgressChangeEvent;
import de.martinknauer.util.progress.ProgressListener;

@SuppressWarnings("serial")
public class UpdaterFrame extends JFrame implements ProgressListener {
	
	private JProgressBar progressBar;

	private JTextArea textArea;

	private Action exitAction;

	private static final Logger logger = LoggerFactory.getLogger(UpdaterFrame.class); 

	public UpdaterFrame(String frameTitle, Action exitAction) {
		
		this.textArea  = new JTextArea(10, 40);
		textArea.setEditable(false);
		textArea.setMargin(new Insets(5, 5, 5, 5));
		textArea.setLineWrap(true);
		textArea.setWrapStyleWord(true);
		
		this.progressBar = new JProgressBar();
		progressBar.setStringPainted(true);
		
		this.exitAction = exitAction;
		
		setTitle(frameTitle);

		setIcon();

		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				exit();
			}
		});
		
		buildInterface();
		
		pack();
		
		Util.locateOnScreen(this);
	}
	
	private void buildInterface() {
		
		JPanel panel = new JPanel(new BorderLayout());
		JPanel innerPanel = new JPanel(new BorderLayout());
		JPanel buttonPanel = new JPanel(new GridBagLayout());
		
		panel.setBorder(BorderFactory.createEmptyBorder(20, 20, 10, 20));
		innerPanel.setBorder(BorderFactory.createEmptyBorder(20, 0, 0, 0));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
		
		JButton button = new JButton(exitAction);
		
		JScrollPane scrollPane = new JScrollPane(this.textArea);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		panel.add(scrollPane, BorderLayout.CENTER);
		panel.add(innerPanel, BorderLayout.SOUTH);
		innerPanel.add(this.progressBar, BorderLayout.CENTER);
		innerPanel.add(buttonPanel, BorderLayout.SOUTH);
		buttonPanel.add(button);
		
		getContentPane().add(panel);
	}
	
	private void setIcon() {

		String iconPath = KreawiGlobals.APP_IMAGES_DIR_NAME + "/trainer_icon.gif";;

		URL url = KreawiGlobals.class.getResource(iconPath);
		if(url != null)
		{
			setIconImage(new ImageIcon(url).getImage());
		}
	}
	
	public void exit() {
		new Thread(new Runnable() {

			public void run() {
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				textArea.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				exitAction.actionPerformed(new ActionEvent(this, 0, ""));
			}
			
		}).start();
		
	}

	public void progressChange(ProgressChangeEvent e) {
		
		if (((int) ((double) progressBar.getValue()) * 100d / ((double) progressBar
				.getMaximum())) != ((int) ((double) e.getNewValue()) * 100d / ((double) e
				.getMaxValue()))) {
			progressBar.setMinimum(e.getMinValue());
			progressBar.setMaximum(e.getMaxValue());
			progressBar.setValue(e.getNewValue());
		}
		
		if (e.getMessage() != null) {
			appendMessage(e.getMessage());
		}
		
//		if (progressBar.getMaximum() <= progressBar.getValue()) {
//			exitAction.setEnabled(true);
//		} else {
//			exitAction.setEnabled(false);
//		}
		
	}
	
	public void appendMessage(String message) {
		textArea.append(message);
		textArea.setCaretPosition(textArea.getText().length());
	}

}
